<?php

namespace Service;

class OrderService {

    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager){
        $this->entityManager = $entityManager;
    }

    public function insertVendaRest($filial, $nxtVendaRest, $store, $vendor, $operator, $client, $consumer){
        $params = array(
            ':CDFILIAL' => $filial,
            ':NRVENDAREST' => $nxtVendaRest,
            ':CDLOJA' => $store,
            ':NRMESA' => '',
            ':CDVENDEDOR' => $vendor,
            ':CDOPERADOR' => $operator,
            ':NRPESMESAVEN' => '1',
            ':CDCLIENTE' => $client,
            ':CDCONSUMIDOR' => $consumer
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_VENDAREST, $params);
    }

    public function insertComandaVen($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $CDLOJA, $IDSTCOMANDA, $SGSEXOCON, $VRACRCOMANDA, $CDPAIS, $SGESTADO, $CDMUNICIPIO, $NRCEPCONSCOMAND, $CDBAIRRO, $DSENDECONSCOMAN, $NRENDECONS, $DSCOMPLENDCOCOM, $DSREFENDCONSCOM, $ISTOGO){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':NRVENDAREST' => $NRVENDAREST,
            ':NRCOMANDA' => $NRCOMANDA,
            ':CDLOJA' => $CDLOJA,
            ':DSCOMANDA' => 'DLV_'.$NRCOMANDA,
            ':IDSTCOMANDA' => $IDSTCOMANDA,
            ':SGSEXOCON' => 'M',
            ':TXMOTIVCANCE' => '',
            ':VRACRCOMANDA' => $VRACRCOMANDA,
            ':CDPAIS' => $CDPAIS,
            ':SGESTADO' => $SGESTADO,
            ':CDMUNICIPIO' => $CDMUNICIPIO,
            ':NRCEPCONSCOMAND' => $NRCEPCONSCOMAND,
            ':CDBAIRRO' => $CDBAIRRO,
            ':DSENDECONSCOMAN' => substr($DSENDECONSCOMAN, 0, 44).', '.$NRENDECONS,
            ':DSCOMPLENDCOCOM' => $DSCOMPLENDCOCOM,
            ':DSREFENDCONSCOM' => $DSREFENDCONSCOM,
            ':IDORGCMDVENDA' => 'DLV_SIT',
            ':IDRETBALLOJA' => $ISTOGO ? 'S' : 'N'
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_COMANDAVEN, $params);
    }

    public function insertProdutoComandaVen($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $CDLOJA){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':NRVENDAREST' => $NRVENDAREST,
            ':NRCOMANDA' => $NRCOMANDA,
            ':CDLOJA' => $CDLOJA,
            ':DSCOMANDA' => 'DLV_'.$NRCOMANDA
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_PRODUTO_COMANDAVEN, $params);
    }

        public function insertProdutoVendaRest($filial, $nxtVendaRest, $store, $vendor, $operator){
        $params = array(
            ':CDFILIAL' => $filial,
            ':NRVENDAREST' => $nxtVendaRest,
            ':CDLOJA' => $store,
            ':CDVENDEDOR' => $vendor,
            ':CDOPERADOR' => $operator
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_PRODUTO_VENDAREST, $params);
    }

    public function insertMovCaixaDlv($CDFILIAL, $NRVENDAREST, $NRSEQMOVDLV, $IDTIPOMOVIVEDLV, $CDCLIENTE, $CDCONSUMIDOR, $CDTIPORECE, $VRMOVIVENDDLV){
        $params = array(
            ':CDFILIAL'         => $CDFILIAL,
            ':NRVENDAREST'      => $NRVENDAREST,
            ':NRSEQMOVDLV'      => $NRSEQMOVDLV,
            ':IDTIPOMOVIVEDLV'  => $IDTIPOMOVIVEDLV,
            ':CDCLIENTE'        => $CDCLIENTE,
            ':CDCONSUMIDOR'     => $CDCONSUMIDOR,
            ':CDTIPORECE'       => $CDTIPORECE,
            ':VRMOVIVENDDLV'    => floatval(str_replace(",", ".", $VRMOVIVENDDLV)),
            ':NRCARTBANCODLV'   => null
        );

        return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_MOVCAIXADLV, $params);
    }

    public function getMoney(){
        $params = array();
        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_MONEY, $params);
        return !Empty($result) ? $result[0] : $result;
    }

    public function checkOrder($CDFILIAL, $DSCOMANDA){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':DSCOMANDA' => $DSCOMANDA
        );
        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::CHECK_ORDER_DETAILS, $params);
        return !Empty($result) ? $result[0] : $result;
    }

    public function getSequenceCode($CDFILIAL, $NRVENDAREST, $NRCOMANDA){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':NRVENDAREST' => $NRVENDAREST,
            ':NRCOMANDA' => $NRCOMANDA
        );
        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_NRSEQPRODCOM, $params);
        return !Empty($result) ? $result[0] : $result;
    }

    public function changeOrderStatus($CDFILIAL, $NRVENDAREST, $NRCOMANDA) {
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':NRVENDAREST' => $NRVENDAREST,
            ':NRCOMANDA' => $NRCOMANDA,
            ':IDSTCOMANDA' => '2'
        );

        $this->entityManager->getConnection()->executeQuery(\Util\Query::SET_ORDER_STATUS, $params);
    }

    public function getOnlinePaymentInfo() {
        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ONLINE_PAYMENT_INFO, array());
        return count($result) > 0 ? $result[0] : $result;
    }

}