<?php

namespace Odhen\API\Service;

class Caixa {

	protected $entityManager;
	protected $util;
	protected $databaseUtil;
    protected $impressaoUtil;
	protected $impressaoSATService;

	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
		\Odhen\API\Util\Util $util,
		\Odhen\API\Util\DataBase $databaseUtil,
    	\Odhen\API\Lib\ImpressaoUtil $impressaoUtil,
    	\Odhen\API\Service\ImpressaoSAT $impressaoSATService) {

	   	$this->entityManager = $entityManager;
		$this->util	= $util;
		$this->databaseUtil = $databaseUtil;
        $this->impressaoUtil = $impressaoUtil;
		$this->impressaoSATService = $impressaoSATService;
	}

	public function abreCaixa($CDFILIAL, $DTABERCAIX, $CDCAIXA, $NRORG, $CDOPERADOR, $DTMOVTURCAIX, $VRMOVIVEND, $IDMONGO, $IDATUTURCAIXA, $flagImprimeAbertura = false, $VALORABERTURA = 0) {
		try {
    		$connection = $this->entityManager->getConnection();
			$connection->beginTransaction();

	    	$DTMOVTURCAIX = $DTMOVTURCAIX . ' 00:00:00';
	      	$VRMOVIVEND = self::removeMascaraDinheiro($VRMOVIVEND);

	      	$estadoCaixa = self::getEstadoCaixa($CDFILIAL, $CDCAIXA);

			if ($estadoCaixa == 'fechado') {
				self::insertTurcaixa($CDFILIAL, $CDCAIXA, $DTABERCAIX, $CDOPERADOR, $DTMOVTURCAIX, $NRORG, $IDMONGO, $IDATUTURCAIXA);

	    		$contador = 'MOVCAIXA' . $CDFILIAL . $CDCAIXA . $DTABERCAIX;

	    		$NRSEQUMOVI = $this->util->geraCodigo($connection, $contador, $NRORG, 1, 10);

				$params = array(
					'NMTIPORECE'=> 'DINHEIRO',
					'NRORG'=> $NRORG
				);
	    		$tiporece = $connection->fetchAll(self::GET_TIPORECE, $params);
	    		$CDTIPORECE = $tiporece[0]['CDTIPORECE'];
				$IDTIPOMOVIVE = 'A';
				$params = array(
					'CDFILIAL' => $CDFILIAL,
					'CDCAIXA' => $CDCAIXA,
					'DTABERCAIX' => $this->databaseUtil->convertToDateTime($DTABERCAIX),
					'IDTIPOMOVIVE' => $IDTIPOMOVIVE,
					'VRMOVIVEND' => $VRMOVIVEND,
					'CDTIPORECE' => $CDTIPORECE,
					'DTHRINCMOV' => $this->databaseUtil->convertToDateTime($DTABERCAIX),
					'DTMOVIMCAIXA' => $this->databaseUtil->getCurrentDate(),
					'NRORG' => $NRORG,
					'NRORGINCLUSAO' => $NRORG,
					'CDOPERADOR' => $CDOPERADOR,
					'CDOPERINCLUSAO' => $CDOPERADOR,
					'IDMONGO' => $IDMONGO,
					'NRSEQUMOVI' => $NRSEQUMOVI,
					'NRSEQUMOVIMSDE' => $NRSEQUMOVI,
					'NRSEQVENDA' => null
				);
				$this->entityManager->getConnection()->executeQuery(self::INSERT_MOVCAIXA, $params);

				if ($flagImprimeAbertura) {
			        $params = array(
			            'CDFILIAL' => $CDFILIAL,
			            'CDCAIXA' => $CDCAIXA,
			            'NRORG' => $NRORG
			        );
			        $dadosImpressora = $this->entityManager->getConnection()->fetchAll(ImpressaoSAT::BUSCA_DADOS_IMPRESSORA, $params);
			        if (!empty($dadosImpressora)) {
			            $IDMODEIMPRES = $dadosImpressora[0]['IDMODEIMPRES'];
			            $CDPORTAIMPR = $dadosImpressora[0]['CDPORTAIMPR'];
			            $CDLOJA = $dadosImpressora[0]['CDLOJA'];
			            $NMLOJA = $dadosImpressora[0]['NMLOJA'];
			            $NMCAIXA = $dadosImpressora[0]['NMCAIXA'];

			            $resposta = $this->impressaoUtil->iniciarPorta($IDMODEIMPRES, $CDPORTAIMPR);
						if ($resposta['error'] == false) {
							$params = array(
								':CDOPERADOR' => $CDOPERADOR
							);
							$NMOPERADOR = $this->entityManager->getConnection()->fetchAssoc(self::GET_NMOPERADOR, $params)['NMOPERADOR'];
							$printerParams = $this->impressaoUtil->buscaParametrosImpressora($IDMODEIMPRES);
							$this->imprimeRelatorioAbertura($CDCAIXA, $NMCAIXA, $CDOPERADOR, $NMOPERADOR, $IDMODEIMPRES, $printerParams, $VALORABERTURA);

							$connection->commit();

							return array(
								'error' => false,
								'message' => "Caixa aberto com sucesso."
							);
						} else {
			                $result = $resposta;
			            }
					} else {
			            $result = array(
			                'error' => true,
			                'message' => 'Falha ao imprimir comprovante de abertura. Impressora não parametrizada para o caixa.'
			            );
			        }
				} else {
					$connection->commit();

					return array(
						'error' => false,
						'message' => "Caixa aberto com sucesso."
					);
				}
			} else {
				return array(
					'error' => true,
					'message' => "O caixa " . $CDCAIXA . " se encontra aberto. Operação inválida."
				);
			}

		} catch(\Exception $e) {
			$this->entityManager->getConnection()->rollBack();
			return array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}
    }

    const GET_TIPORECE = "
    	SELECT CDTIPORECE
    	  FROM TIPORECE
    	 WHERE UPPER(NMTIPORECE) = :NMTIPORECE
    	   AND NRORG = :NRORG
    ";
    const GET_NMOPERADOR = "
    	SELECT NMOPERADOR
    	  FROM OPERADOR
    	 WHERE CDOPERADOR = :CDOPERADOR
    ";

    const INSERT_MOVCAIXA = "
		INSERT INTO MOVCAIXA
		(CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQUMOVI,
		NRSEQUMOVIMSDE, DTHRINCMOV,	IDTIPOMOVIVE, VRMOVIVEND,
		NRSEQVENDA, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO,
		CDTIPORECE, DTMOVIMCAIXA)
		VALUES
		(:CDFILIAL, :CDCAIXA, :DTABERCAIX, :NRSEQUMOVI,
		:NRSEQUMOVIMSDE, :DTHRINCMOV, :IDTIPOMOVIVE, REPLACE(:VRMOVIVEND,'.',','),
		:NRSEQVENDA, :NRORG, :NRORGINCLUSAO, :CDOPERINCLUSAO,
		:CDTIPORECE, :DTMOVIMCAIXA)
    ";

    private function removeMascaraDinheiro($VRMOVIVEND) {
    	$VRMOVIVEND = str_replace('R$', '', $VRMOVIVEND);
		$VRMOVIVEND = str_replace('.', '', $VRMOVIVEND);
		$VRMOVIVEND = str_replace(',', '.', $VRMOVIVEND);
		return floatval($VRMOVIVEND);
    }

    private function insertTurcaixa($CDFILIAL, $CDCAIXA, $DTABERCAIX, $CDOPERADOR, $DTMOVTURCAIX, $NRORG, $IDMONGO, $IDATUTURCAIXA) {
    	$params = array(
    		'CDFILIAL' => $CDFILIAL,
    		'CDCAIXA' => $CDCAIXA,
    		'DTABERCAIX' => $this->databaseUtil->convertToDateTime($DTABERCAIX),
    		'CDOPERADOR' => $CDOPERADOR,
    		'DTMOVTURCAIX' => $this->databaseUtil->convertToDateTime($DTMOVTURCAIX),
    		'NRORG' => $NRORG,
    		'IDMONGO' => $IDMONGO,
    		'IDATUTURCAIXA' => $IDATUTURCAIXA
		);

		$this->entityManager->getConnection()->executeQuery(self::INSERT_TURCAIXA, $params);
    }

    const INSERT_TURCAIXA = "
	    INSERT INTO TURCAIXA
	    (CDFILIAL, CDCAIXA, DTABERCAIX, CDOPERADOR,
	    CDOPERABER, DTMOVTURCAIX, NRORG, IDATUTURCAIXA)
	    VALUES
	    (:CDFILIAL, :CDCAIXA, :DTABERCAIX, :CDOPERADOR,
	    :CDOPERADOR, :DTMOVTURCAIX, :NRORG, :IDATUTURCAIXA)
    ";

	public function fechaCaixa($CLOSEPOSSIMPLE, $DTABERCAIX, $DTFECHCAIX, $CDFILIAL, $CDCAIXA, $NRORG, $CDOPERFECH, $NRCONFTELA = null) {
		$retornoFechaCaixa = array();
		
		try {
			$this->entityManager->getConnection()->beginTransaction();

			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'CDCAIXA'  => $CDCAIXA
			);

			$estadoCaixa = self::getEstadoCaixa($CDFILIAL, $CDCAIXA);
			if ($estadoCaixa == 'aberto') {
				if ($DTABERCAIX == null) {
					$DTABERCAIX = self::getUltimaAberturaCaixa($CDFILIAL, $CDCAIXA);
				}

				$IDATUTURCAIXA = 'A';

				if ($CLOSEPOSSIMPLE == 'S') {
					$dadosMovCaixa = self::preparaParamsMovCaixa($DTABERCAIX, $DTFECHCAIX, $CDFILIAL, $CDCAIXA, $NRORG, $NRCONFTELA);

					// Insere uma linha na movcaixa para cada tipo de recebimento de sangria automática
					foreach ($dadosMovCaixa as $paramsMovCaixa) {
						if ($this->databaseUtil->databaseIsOracle()) {
							$this->entityManager->getConnection()->executeQuery(self::INSERT_MOVCAIXA_OPENPOS_ORACLE, $paramsMovCaixa);
						} else {
							$this->entityManager->getConnection()->executeQuery(self::INSERT_MOVCAIXA_OPENPOS_MSDE, $paramsMovCaixa);
						}
					}
				}
				$params = self::preparaParamsTurcaixa($CDOPERFECH, $DTABERCAIX, $DTFECHCAIX, $CDFILIAL, $CDCAIXA, $NRORG, $IDATUTURCAIXA);

				if ($this->databaseUtil->databaseIsOracle()) {
					$this->entityManager->getConnection()->executeQuery(self::UPDATE_DTFECHAMENTO_TURCAIXA_ORACLE, $params);
				} else {
					$this->entityManager->getConnection()->executeQuery(self::UPDATE_DTFECHAMENTO_TURCAIXA_MSDE, $params);
				}

				$this->entityManager->getConnection()->commit();
				$retornoFechaCaixa = array(
					'error' => false,
					'message' => "Caixa fechado com sucesso."
				);
			} else {
				$retornoFechaCaixa = array(
					'error' => true,
					'message' => "Não existe um caixa aberto para executar o fechamento. Operação inválida."
				);
			}
		} catch(\Exception $e) {
			$this->entityManager->getConnection()->rollBack();
			$retornoFechaCaixa = array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}

		return $retornoFechaCaixa;
    }

    const INSERT_MOVCAIXA_OPENPOS_ORACLE = "
		INSERT INTO MOVCAIXA
		(CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQUMOVI,
		NRSEQUMOVIMSDE, DTHRINCMOV, IDTIPOMOVIVE, VRMOVIVEND,
		CDTIPORECE, DTMOVIMCAIXA, NRORG)
		VALUES
		(:CDFILIAL, :CDCAIXA, :DTABERCAIX, :NRSEQUMOVI,
		:NRSEQUMOVIMSDE, :DTHRINCMOV, :IDTIPOMOVIVE, :VRMOVIVEND,
		:CDTIPORECE, TO_DATE(:DTMOVIMCAIXA, 'DD/MM/YYYY'), :NRORG)
    ";

    const INSERT_MOVCAIXA_OPENPOS_MSDE = "
		INSERT INTO MOVCAIXA
		(CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQUMOVI,
		NRSEQUMOVIMSDE, DTHRINCMOV, IDTIPOMOVIVE, VRMOVIVEND,
		CDTIPORECE, DTMOVIMCAIXA, NRORG)
		VALUES
		(:CDFILIAL, :CDCAIXA, :DTABERCAIX, :NRSEQUMOVI,
		:NRSEQUMOVIMSDE, :DTHRINCMOV, :IDTIPOMOVIVE, :VRMOVIVEND,
		:CDTIPORECE, :DTMOVIMCAIXA, :NRORG)
    ";

    const UPDATE_DTFECHAMENTO_TURCAIXA_MSDE = "
	    UPDATE TURCAIXA
	       SET DTFECHCAIX = :DTFECHCAIX,
	           CDOPERFECH = :CDOPERFECH,
	           CDOPERULTATU = :CDOPERFECH,
	           NRORGULTATU = :NRORG,
	           IDATUTURCAIXA = :IDATUTURCAIXA
	     WHERE CDFILIAL =:CDFILIAL
	       AND CONVERT(DATETIME, DTABERCAIX, 120) = CONVERT(DATETIME, :DTABERCAIX, 120)
	       AND CDCAIXA = :CDCAIXA
	       AND NRORG = :NRORG
    ";

    const UPDATE_DTFECHAMENTO_TURCAIXA_ORACLE = "
	    UPDATE TURCAIXA
	       SET DTFECHCAIX = :DTFECHCAIX,
	           CDOPERFECH = :CDOPERFECH,
	           CDOPERULTATU = :CDOPERFECH,
	           NRORGULTATU = :NRORG,
	           IDATUTURCAIXA = :IDATUTURCAIXA
	     WHERE CDFILIAL =:CDFILIAL
	       AND DTABERCAIX = :DTABERCAIX
	       AND CDCAIXA = :CDCAIXA
	       AND NRORG = :NRORG
    ";

    private function preparaParamsMovCaixa($DTABERCAIX, $DTFECHCAIX, $CDFILIAL, $CDCAIXA, $NRORG, $NRCONFTELA) {
		$DTMOVTURCAIX = $DTABERCAIX;
		$connection = $this->entityManager->getConnection();
	
		$dadosMovCaixa = array();

		$contador = 'MOVCAIXA' . $CDFILIAL . $CDCAIXA . $DTABERCAIX;

		$dadosSangriaAutomatica = self::buscaSangriaAutomatica($CDFILIAL, $CDCAIXA, $DTABERCAIX, $NRCONFTELA);
		foreach ($dadosSangriaAutomatica as $sangria) {
			/*
			* G -> Sangria (Fechamento do caixa)
			* A -> Abertura
			* S -> Saída
			* E -> Entrada
			*/
			$IDTIPOMOVIVE = 'G';
			$NRSEQUMOVI = $this->util->geraCodigo($connection, $contador, $NRORG, 1, 10);
			$linhaMovCaixa = array(
				'CDFILIAL' => $CDFILIAL,
				'CDCAIXA' => $CDCAIXA,
				'DTABERCAIX' => $this->databaseUtil->convertToDateTime($DTABERCAIX),
				'NRSEQUMOVI' => $NRSEQUMOVI,
				'NRSEQUMOVIMSDE' => $NRSEQUMOVI,
				'IDTIPOMOVIVE' => $IDTIPOMOVIVE,
				'VRMOVIVEND' => $sangria['VRMOVIVEND'],
				'CDTIPORECE' => $sangria['CDTIPORECE'],
				'DTHRINCMOV' => $this->databaseUtil->convertToDateTime($DTFECHCAIX),
				'DTMOVIMCAIXA' => $this->databaseUtil->convertToDate($DTMOVTURCAIX),
				'NRORG' => $NRORG
			);

			array_push($dadosMovCaixa, $linhaMovCaixa);
		}

		return $dadosMovCaixa;
	}

	private function preparaParamsTurcaixa($CDOPERFECH, $DTABERCAIX, $DTFECHCAIX, $CDFILIAL, $CDCAIXA, $NRORG, $IDATUTURCAIXA) {
		return array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'DTFECHCAIX' => $this->databaseUtil->convertToDateTime($DTFECHCAIX),
			// Update this (only working for sql server)
			'DTABERCAIX' => $this->databaseUtil->convertToDateTime($DTABERCAIX)->format('Y/m/d H:i:s'),
			'CDOPERFECH' => $CDOPERFECH,
			'NRORG' => $NRORG,
			'IDATUTURCAIXA' => $IDATUTURCAIXA
		);
	}

	public function getEstadoCaixa($CDFILIAL, $CDCAIXA) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA'  => $CDCAIXA
		);

		$estadoCaixa = $this->entityManager->getConnection()->fetchAll(self::VERIFICA_ABERTURA_CAIXA, $params);

		if(empty($estadoCaixa) == true) {
			return 'fechado';
		} else {
			return 'aberto';
		}
	}

	const VERIFICA_ABERTURA_CAIXA = "
    	SELECT DTFECHCAIX
    	FROM TURCAIXA
    	WHERE CDFILIAL = :CDFILIAL
    		AND CDCAIXA = :CDCAIXA
    		AND DTFECHCAIX IS NULL
    ";

    const GET_ULTIMA_ABERTURA_CAIXA = "
    	SELECT DTABERCAIX
    	FROM TURCAIXA
    	WHERE CDFILIAL = :CDFILIAL
    		AND CDCAIXA = :CDCAIXA
    		AND DTFECHCAIX IS NULL
    ";

    public function getUltimaAberturaCaixa($CDFILIAL, $CDCAIXA) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA'  => $CDCAIXA
		);

		$DTABERCAIX = $this->entityManager->getConnection()->fetchAll(self::GET_ULTIMA_ABERTURA_CAIXA, $params);
		$DTABERCAIX = explode('.', $DTABERCAIX[0]['DTABERCAIX'])[0];
		$DTABERCAIX = \DateTime::createFromFormat('Y-m-d H:i:s', $DTABERCAIX)->format('d/m/Y H:i:s');

		return $DTABERCAIX;
    }

    public function imprimeRelatorioAbertura($CDCAIXA, $NMCAIXA, $CDOPERADOR, $NMOPERADOR, $CDOPERADOR, $IDMODEIMPRES, $printerParams, $VALORABERTURA = 0){
		$texto .= $this->impressaoUtil->centraliza($printerParams, 'COMPROVANTE DE ABERTURA') . $printerParams['comandoEnter'];
		$texto .= $this->impressaoUtil->preencheLinha($printerParams, '', '-');
		$texto .= $this->impressaoUtil->centraliza($printerParams, 'CAIXA      '.$CDCAIXA. ' - '. $NMCAIXA) . $printerParams['comandoEnter'];
		$now =  $this->databaseUtil->getCurrentDateTime();
		$texto .= $this->impressaoUtil->centraliza($printerParams, 'DATA       '.$now->format('Y/m/d H:i:s')) . $printerParams['comandoEnter'];
		$texto .= $this->impressaoUtil->centraliza($printerParams, 'OPERADOR   '.$CDOPERADOR.' - '.$NMOPERADOR) . $printerParams['comandoEnter'];
		$texto .= $this->impressaoUtil->preencheLinha($printerParams, '', '-');
		if($VALORABERTURA == 0){
			$texto .= $this->impressaoUtil->centraliza($printerParams, 'Caixa aberto sem suprimento') . $printerParams['comandoEnter'];
		}else{
			$texto .= '     Suprimento                     '.number_format($VALORABERTURA, 2) . $printerParams['comandoEnter'];
			$texto .= 'TOTAL        R$                     '.number_format($VALORABERTURA, 2) . $printerParams['comandoEnter'];
		}
		$this->impressaoUtil->imprimeTexto($texto, $IDMODEIMPRES);
    }

    public function imprimeRelatorioFechamento($CDFILIAL, $DTABERCAIX, $CDCAIXA, $CDOPERADOR, $NRORG, $flagReducaoZ = true, $flagFinal = true){
        try {
            $this->entityManager->getConnection()->beginTransaction();
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDCAIXA' => $CDCAIXA,
                'NRORG' => $NRORG
            );
            $dadosImpressora = $this->entityManager->getConnection()->fetchAll(ImpressaoSAT::BUSCA_DADOS_IMPRESSORA, $params);
            if (empty($dadosImpressora)) {
                return array(
                    'error' => true,
                    'message' => 'Falha ao imprimir comprovante de abertura. Impressora não parametrizada para o caixa.'
                );
            } else {
                $IDMODEIMPRES = $dadosImpressora[0]['IDMODEIMPRES'];
                $CDPORTAIMPR = $dadosImpressora[0]['CDPORTAIMPR'];
                $params = array(
                    ':CDFILIAL' => $CDFILIAL,
                    ':NRORG' => $NRORG
                );

                $printerParams = $this->impressaoUtil->buscaParametrosImpressora($IDMODEIMPRES);
                $dadosFilial = $this->entityManager->getConnection()->fetchAll(ImpressaoSAT::GET_DADOS_FILIAL, $params);
                $NMRAZSOCFILI = $dadosFilial[0]['NMRAZSOCFILI'];
                $NRINSJURFILI = $dadosFilial[0]['NRINSJURFILI'];
                $DSENDEFILI = $dadosFilial[0]['DSENDEFILI'];
                $NMBAIRFILI = $dadosFilial[0]['NMBAIRFILI'];
                $NMMUNICIPIO = $dadosFilial[0]['NMMUNICIPIO'];
                $SGESTADO = $dadosFilial[0]['SGESTADO'];
                $CDINSCESTA = $dadosFilial[0]['CDINSCESTA'];
                $NMFILIAL = $dadosFilial[0]['NMFILIAL'];
                $CDINSCMUNI = $dadosFilial[0]['CDINSCMUNI'];
                $texto = '';
                $texto .= $this->impressaoSATService->cabecalhoCupomNF($printerParams, $CDFILIAL, $CDCAIXA, $DTABERCAIX, $NMRAZSOCFILI, $NRINSJURFILI, $DSENDEFILI, $NMBAIRFILI, $NMMUNICIPIO, $SGESTADO, $CDINSCESTA, $CDINSCMUNI);
                $params = array(
                    ':CDOPERADOR' => $CDOPERADOR
                );
                $NMOPERADOR = $this->entityManager->getConnection()->fetchAssoc(self::GET_NMOPERADOR, $params)['NMOPERADOR'];
                if($flagReducaoZ == true && $flagFinal == true){
                    $texto .= $this->impressaoUtil->centraliza($printerParams, 'RELATORIO FECHAMENTO DE CAIXA - TOTAL DIA') . $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
                }elseif($flagReducaoZ == true){
                    $texto .= $this->impressaoUtil->centraliza($printerParams, 'RELATORIO FECHAMENTO DE CAIXA') . $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
                }else{
                    $texto .= $this->impressaoUtil->centraliza($printerParams, 'RELATORIO PARCIAL DIA') . $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
                }
                $ArrayMessagesBrasil = array(
                    'DATA' => 'DATA',
                    'HORA' => 'HORA',
                    'FILIAL' => 'FILIAL',
                    'OPERADOR' => 'OPERADOR',
                    'CAIXA' => 'CAIXA',
                    'PRODUTO' => 'PRODUTO',
                    'MOTIVO' => 'MOTIVO',
                    'MESA' => 'MESA',
                    'GARCOM' => 'GARCOM',
                    'CUPOM' => 'NR.CUPOM',
                    'RECIBOSAIDANUMERARIO' => 'RECIBO SAIDA NUMERARIO',
                    'CANCPEDIDO' => 'CANCELAMENTO DE PEDIDO',
                    'CONFMESA' => 'CONFERENCIA DE MESA',
                    'EMISSAO' => 'EMISSAO',
                    'COMANDA' => 'COMANDA\MESA',
                    'CONSUMAMINIMA' => 'CONSUMACAO MINIMA',
                    'COUVERT' => 'COUVERT',
                    'TAXASERVICO' => 'TAXA SERVICO',
                    'TOTAL' => 'TOTAL',
                    'DOCSEMVALOR' => 'DOC. SEM VALOR FISCAL',
                    'DESCONTO' => 'DESCONTO',
                    'SUBTOTAL' => 'SUBTOTAL',
                    'RECEBIMENTO' => 'RECEBIMENTO',
                    'TROCO' => 'TROCO',
                    'MATRICULA' => 'MATRICULA',
                    'CONSUMIDOR' => 'CONSUMIDOR',
                    'CREDITOPESSOAL' => 'VENDA - CREDITO PESSOAL',
                    'INSCMUNICIPAL' => 'INSC. MUNICIPAL',
                    'ACRESCIMO' => 'ACRESCIMO',
                    'AMBIENTE' => 'AMBIENTE',
                    'QUANT' => 'QUANT',
                    'TOTALACRESCIMOS' => 'TOTAL ACRESCIMOS',
                    'TOTALDESCONTOS' => 'TOTAL DESCONTOS',
                    'TOTALDASVENDAS' => 'TOTAL DAS VENDAS',
                    'SAIDANUMERARIOSANGRIA' => 'SAIDA DE NUMERARIO - SANGRIA',
                    'SUPERVISOR' => 'SUPERVISOR',
                    'TEKNISA' => 'TEKNISA SOFTWARE -  www.teknisa.com',
                    'TOTALPRODUTOS' => 'TOTAL PRODUTOS',
                    'PESSOAS' => 'PESSOA(S)',
                    'TOTALGERAL' => 'TOTAL GERAL',
                    'PORPESSOA' => 'POR PESSOA',
                    'CONFCOMANDA' => 'CONFERENCIA DE COMANDA',
                    'ABERTURA' => 'ABERTURA',
                    'RELPRODUTODIA' => 'RELATORIO DE PRODUTOS - DIA',
                    'RELPRODUTOABER' => 'RELATORIO DE PRODUTOS - ABERTURA',
                    'TOTALPRODUTO' => 'TOTAL PRODUTOS',
                    'NRSERIE' => 'NR.SERIE',
                    'DATEREDUCAO' => 'DATA REDUCAO',
                    'REDUCAOZ' => 'REDUCAO Z',
                    'LEITURAX' => 'LEITURA X',
                    'VENDABRUTA' => 'VENDA BRUTA',
                    'QTDEPRODUTOS' => 'QUANT PRODUTOS',
                    'QTDEVENDAS' => 'QUANT DE VENDAS',
                    'VENDALIQUIDA' => 'VENDA LIQUIDA',
                    'QTDECANCELADA' => 'QUANT CANCELADAS',
                    'VENDASCANCELADAS' => 'VENDAS CANCELADAS',
                    'BASEIMPOSTO' => 'BASE IMPOSTO',
                    'CAIXASVENDAS' => 'CAIXAS(VENDAS POR ABERTURA DE CAIX)',
                    'QTDEPRODUTO' => 'QUANT. PRODUTOS',
                    'DATAREDUCAO' => 'DATA REDUCAO',
                    'FORMAPAGAMENTO' => 'FORMA DE PAGAMENTO',
                    'VALOR' => 'VALOR',
                    'POSICAO' => 'POSICAO ',
                    'TOTALFECHA' => 'TOTAL DE FECHAMENTO DE CAIXA (Z)',
                    'TOTALPARCIAL' => 'TOTAL PARCIAL DE CAIXA (X)',
                    'INSCRICAOMUNICIPAL' => 'Inscricao Municipal',
                    'ComprDebito' => 'Venda - Debito Consumidor',
                    'CDCLIE' => 'Cliente: ',
                    'Data1' => 'Data:',
                    'CDCons' => 'Matricula: ',
                    'NmCons' => 'Consumidor..: ',
                    'NmFami' => 'Familia.....: ',
                    'SEN_PED' => 'SENHA PEDIDO:  ',
                    'SALDO_FIN' => 'Saldo Final : %s',
                    'PRECO_SUBSIDIO' => 'Preço subsidiado:',
                    'SENHA' => 'SENHA:',
                    'Erro_NF' => 'Erro de impressão na impressora não fiscal.',
                    'Erro_Porta' => 'Não foi possível abrir a porta de comunicação da impressora não fiscal.',
                    'VrCupom' => 'Valor ......: ',
                    'VrSaldo' => 'Saldo Final : ',
                    'ComprCredito' => 'COMPRA DE CREDITO',
                    'Nr_Deposito' => 'Nr.Deposito: ',
                    'FORMAPAGO' => 'FORMAS DE PAGAMENTO - ABERTURA');


                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, $ArrayMessagesBrasil['FILIAL']   , $CDFILIAL   . $printerParams['comandoEnter']);
                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, $ArrayMessagesBrasil['CAIXA']    , $CDCAIXA    . $printerParams['comandoEnter']);
                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, $ArrayMessagesBrasil['DATA']     , $DTABERCAIX . $printerParams['comandoEnter']);
                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, $ArrayMessagesBrasil['OPERADOR'] , $CDOPERADOR.' - '.$NMOPERADOR. $printerParams['comandoEnter']);
                $texto .= $this->impressaoUtil->preencheLinha($printerParams, '', '-');

                $params = array(
                    ':CDFILIAL'   => $CDFILIAL,
                    ':CDCAIXA'    => $CDCAIXA,
                    ':DTABERTUR'  => $this->databaseUtil->convertToDateTime($DTABERCAIX),
                    ':DTABERCAIX' => $this->databaseUtil->convertToDateTime($DTABERCAIX)
                );
                $VRVENDBRUT = $this->entityManager->getConnection()->fetchAssoc(self::GET_VRVENDBRUT, $params)['VRVENDBRUT'];
                $VRTOTINI = $this->entityManager->getConnection()->fetchAssoc(self::GET_VRTOTINI, $params)['VRTOTINI'];
                $NRTRANSACOES = $this->entityManager->getConnection()->fetchAssoc(self::GET_NRTRANSACOES, $params)['NRTRANSACOES'];
                $NRPRIMSEQ = $this->entityManager->getConnection()->fetchAssoc(self::GET_NRPRIMSEQ, $params)['NRPRIMSEQ'];
                $NRFINALSEQ = $this->entityManager->getConnection()->fetchAssoc(self::GET_NRFINALSEQ, $params)['NRFINALSEQ'];

                $CANCELAMENTOS = $this->entityManager->getConnection()->fetchAssoc(self::GET_CANCELAMENTOS, $params);
                $TRANSCANC = $CANCELAMENTOS['TRANSCANC'];
                $VRCANCEL = $CANCELAMENTOS['VRCANCEL'];

                if($flagFinal == true){
                    $VRDESITVEND = $this->entityManager->getConnection()->fetchAssoc(self::GET_VRDESITVENDFIN, $params)['VRDESITVENDFIN'];
                    $VRACRITVEND = $this->entityManager->getConnection()->fetchAssoc(self::GET_VRACRITVENDFIN, $params)['VRACRITVENDFIN'];
                    $VRMOVIVEND  = $this->entityManager->getConnection()->fetchAssoc(self::GET_VRMOVIVENDFIN, $params)['VRMOVIVENDFIN'];
                }else{
                    $VRDESITVEND = $this->entityManager->getConnection()->fetchAssoc(self::GET_VRDESITVEND, $params)['VRDESITVEND'];
                    $VRACRITVEND = $this->entityManager->getConnection()->fetchAssoc(self::GET_VRACRITVEND, $params)['VRACRITVEND'];
                    $VRMOVIVEND  = $this->entityManager->getConnection()->fetchAssoc(self::GET_VRMOVIVEND, $params)['VRMOVIVEND'];
                }

                if($flagFinal == true){
                    $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, 'TOTALIZADOR INICIAL', number_format($VRTOTINI - $VRVENDBRUT , 2));
                    $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, 'TOTALIZADOR FINAL', number_format($VRTOTINI , 2));

                    $texto .= $this->impressaoUtil->preencheLinha($printerParams, '', '-');
                }

                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, 'NR. DE TRANSACOES CONCLUIDAS' , $NRTRANSACOES);
                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, 'NR. DO CUPOM INICIAL' , $NRPRIMSEQ);
                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, 'NR. DO CUPOM FINAL' , $NRFINALSEQ);
                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, 'NR. CUPONS CANCELADOS' , $TRANSCANC);

                $texto .= $this->impressaoUtil->preencheLinha($printerParams, '', '-');

                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, '(1)VALOR VENDA BRUTA' , number_format($VRVENDBRUT,2));
                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, '(2)VALOR DE CANCELAMENTO' , number_format($VRCANCEL,2));
                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, '(3)VALOR TOTAL DESCONTOS' , number_format($VRDESITVEND,2));
                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, '(4)VALOR TOTAL ACRESCIMOS' , number_format($VRACRITVEND,2));
                $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, 'VALOR VENDA LIQUIDA (1-2-3)' , number_format($VRVENDBRUT-$VRDESITVEND-$VRCANCEL,2));

                $texto .= $this->impressaoUtil->preencheLinha($printerParams, '', '-');

                if($flagFinal == true)
                    $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, 'VALOR FUNDO DE TROCO - TOTAL DIA' , number_format($VRMOVIVEND,2));
                else {
                    $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, 'VALOR FUNDO DE TROCO' , number_format($VRMOVIVEND,2));
                }

                $texto .= $this->impressaoUtil->preencheLinha($printerParams, '', '-');


                $IMPOSTOS  = $this->entityManager->getConnection()->fetchAll(self::GET_IMPOSTOS, $params);
                $quarter = floor($printerParams['largura']/4 -2);
                $texto .= $this->impressaoUtil->centraliza($printerParams, 'IMPOSTOS') . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->preenche($quarter,  'SIGLA', ' ');
                $texto .= $this->impressaoUtil->preenche($quarter,  'ALIQ', ' ');
                $texto .= $this->impressaoUtil->preenche($quarter,  'VR.BASE', ' ');
                $texto .= $this->impressaoUtil->preenche($quarter,  'VR.IMP', ' '). $printerParams['comandoEnter'];

                foreach ($IMPOSTOS as $imposto) {
                    $texto .= $this->impressaoUtil->preenche($quarter, $imposto['SGIMPOSTO'], ' ');
                    $texto .= $this->impressaoUtil->preenche($quarter, number_format($imposto['VRPEALPRODIT'],3), ' ');
                    $texto .= $this->impressaoUtil->preenche($quarter, number_format($imposto['VRBASE'],3), ' ');
                    $texto .= $this->impressaoUtil->preenche($quarter, number_format($imposto['VRIMPOSTO'],3), ' '). $printerParams['comandoEnter'];
                }

                $texto .= $this->impressaoUtil->preencheLinha($printerParams, '', '-'). $printerParams['comandoEnter'];


                if($flagFinal == true){
                    $PAGAMENTOS  = $this->entityManager->getConnection()->fetchAll(self::GET_PAGAMENTOSFIN, $params);
                    $texto .= $this->impressaoUtil->centraliza($printerParams, 'MEIOS DE PAGAMENTO RECEBIDOS - TOTAL DIA') . $printerParams['comandoEnter'];
                }else{
                    $PAGAMENTOS  = $this->entityManager->getConnection()->fetchAll(self::GET_PAGAMENTOS, $params);
                    $texto .= $this->impressaoUtil->centraliza('MEIOS DE PAGAMENTO RECEBIDOS', tamLinha). $printerParams['comandoEnter'];
                }
                foreach ($PAGAMENTOS as $pagamento) {
                    $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, $pagamento['NMTIPORECE'] , number_format($pagamento['VALOR_TOTAL'], 2));
                    if ($pagamento['VRTROCO'] > 0){
                        $texto .= $this->impressaoUtil->alinhaInicioFim($printerParams, 'TROCO - '.$pagamento['NMTIPORECE'] , number_format($pagamento['VRTROCO'], 2));
                    }
                }
                $now =  $this->databaseUtil->getCurrentDateTime();
                $texto .= $this->impressaoUtil->preencheLinha($printerParams, '', '-');
                $texto .= $this->impressaoUtil->centraliza($printerParams, $ArrayMessagesBrasil['DATA'].' '.$now->format('Y/m/d')  . ' ' . $ArrayMessagesBrasil['HORA'] .' '. $now->format('H:i:s')) . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->centraliza($printerParams, 'Teknisa Software - www.teknisa.com');
                $texto .= $this->impressaoUtil->preencheLinha($printerParams, '', ' ');

                $retorno = $this->impressaoUtil->iniciarPorta($IDMODEIMPRES, $CDPORTAIMPR);
                if ($retorno['error'] == false) {
                	$portaUsbKiosk = $retorno['portaUsbKiosk'];
                    $result = $this->impressaoUtil->imprimeTexto($texto, $IDMODEIMPRES, $portaUsbKiosk);
                    $this->impressaoUtil->cortaPapel($IDMODEIMPRES, $portaUsbKiosk);
                    if ($result['error']){
                        return $result;
                    } else {
                        return array(
                            'error' => false,
                            'message' => 'Relatório impresso com sucesso.'
                        );
                    }
                } else {
                    return $retorno;
                }
            }
        } catch(\Exception $e) {
            $this->entityManager->getConnection()->rollBack();
            return array(
                'error' => true,
                'message' => $e->getMessage()
            );
        }
    }

	function buscaSangriaAutomatica($CDFILIAL, $CDCAIXA, $DTABERCAIX, $NRCONFTELA) {
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':CDCAIXA' => $CDCAIXA,
			':DTABERCAIX' => $DTABERCAIX,
			':NRCONFTELA' => $NRCONFTELA,
		);
		
		$sangriaAutomatica = $this->entityManager->getConnection()->fetchAll(self::GET_SANGRIA_AUTOMATICA, $params);

		return $sangriaAutomatica;
	}

	const GET_SANGRIA_AUTOMATICA = "
		SELECT A.CDTIPORECE, A.NMTIPORECE, A.IDTIPORECE, 
			ISNULL(A.VRMOVIVEND,0) -  ISNULL(B.VRMOVIVEND,0) AS VRMOVIVEND
		FROM (SELECT
				T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE, SUM(M.VRMOVIVEND) AS VRMOVIVEND
				FROM
				MOVCAIXA M, TIPORECE T, ITMENUCONFTE I
				WHERE M.CDFILIAL      = :CDFILIAL
				AND M.CDCAIXA       = :CDCAIXA
				AND (M.IDTIPOMOVIVE  IN ('E','T','U'))
				AND M.DTABERCAIX    = :DTABERCAIX
				AND M.CDTIPORECE    = T.CDTIPORECE
				AND T.IDSANGRIAAUTO = 'S'
				AND M.CDFILIAL      = I.CDFILIAL
				AND M.CDTIPORECE    = I.CDIDENTBUTON
				AND I.NRCONFTELA    = :NRCONFTELA
				AND T.IDTIPORECE    <> '7'
				AND T.IDTIPORECE    <> '5'
				GROUP BY
				T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE) A LEFT JOIN
				(SELECT
				T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE, SUM(M.VRMOVIVEND) AS VRMOVIVEND
				FROM
				MOVCAIXA M, TIPORECE T, ITMENUCONFTE I
				WHERE M.CDFILIAL      = :CDFILIAL
				AND M.CDCAIXA       = :CDCAIXA
				AND M.IDTIPOMOVIVE  = 'S'
				AND M.DTABERCAIX    = :DTABERCAIX
				AND M.CDTIPORECE    = T.CDTIPORECE
				AND T.IDSANGRIAAUTO = 'S'
				AND M.CDFILIAL      = I.CDFILIAL
				AND M.CDTIPORECE    = I.CDIDENTBUTON
				AND I.NRCONFTELA    = :NRCONFTELA
				AND T.IDTIPORECE    <> '7'
				AND T.IDTIPORECE    <> '5'
				GROUP BY
				T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE) B
				ON (A.CDTIPORECE = B.CDTIPORECE)
				UNION ALL
				SELECT
				T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE, SUM(M.VRMOVIVEND) AS VRMOVIVEND
				FROM
				MOVCAIXA M, TIPORECE T, ITMENUCONFTE I
				WHERE M.CDFILIAL      = :CDFILIAL
				AND M.CDCAIXA       = :CDCAIXA
				AND M.IDTIPOMOVIVE  = 'E'
				AND M.DTABERCAIX    = :DTABERCAIX
				AND M.CDTIPORECE    = T.CDTIPORECE
				AND T.IDSANGRIAAUTO = 'S'
				AND M.CDFILIAL      = I.CDFILIAL
				AND M.CDTIPORECE    = I.CDIDENTBUTON
				AND I.NRCONFTELA    = :NRCONFTELA
				AND ((T.IDTIPORECE   = '7') OR (T.IDTIPORECE = '5'))
		GROUP BY T.CDTIPORECE, T.NMTIPORECE, T.IDTIPORECE
	";

	const GET_VRVENDBRUT =
	  "SELECT SUM(ISNULL(A.VENDAS,0)) AS VRVENDBRUT
		 FROM (SELECT I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA,
		              SUM(ROUND(I.QTPRODVEND * (I.VRUNITVEND + ISNULL(I.VRUNITVENDCL,0)),2,1) + ISNULL(I.VRACRITVEND,0)) AS VENDAS
		         FROM ITEMVENDA I JOIN VENDA V ON I.CDFILIAL   = V.CDFILIAL
                                              AND I.CDCAIXA    = V.CDCAIXA
                                              AND I.NRSEQVENDA = V.NRSEQVENDA
		        WHERE I.CDFILIAL   = :CDFILIAL
		          AND I.CDCAIXA    = :CDCAIXA
		          AND CONVERT(VARCHAR,V.DTABERTUR,103) = CONVERT(VARCHAR,:DTABERTUR,103)
		          AND V.IDSITUVENDA IN ('O','C')
		     GROUP BY I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA) A" ;

	const GET_VRTOTINI =
         "SELECT SUM(ISNULL(A.VENDAS,0)+ISNULL(B.VENDAS,0)) AS VRTOTINI
            FROM (SELECT I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA, SUM(ROUND(QTPRODVEND*(VRUNITVEND+ISNULL(VRUNITVENDCL,0)),2,1)+ISNULL(VRACRITVEND,0)) AS VENDAS
                    FROM ITEMVENDA I JOIN VENDA V ON I.CDFILIAL   = V.CDFILIAL
                                                 AND I.CDCAIXA    = V.CDCAIXA
                                                 AND I.NRSEQVENDA = V.NRSEQVENDA
                   WHERE I.CDFILIAL   = :CDFILIAL
                     AND I.CDCAIXA    = :CDCAIXA
                     AND V.IDSITUVENDA IN ('O','C')
    	        GROUP BY I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA) A
       LEFT JOIN
    		         (SELECT I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA, SUM(ROUND(QTPRODVENDC*(VRUNITVENDC+ISNULL(VRUNITVENCLC,0)),2,1)+ISNULL(VRACRITVENDC,0)) AS VENDAS
                    FROM ITVENDACAN I JOIN VENDA V  ON I.CDFILIAL   = V.CDFILIAL
    		       							    AND I.CDCAIXA    = V.CDCAIXA
    		       							    AND I.NRSEQVENDA = V.NRSEQVENDA
                   WHERE I.CDFILIAL   = :CDFILIAL
                     AND I.CDCAIXA    = :CDCAIXA
                GROUP BY I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA) B
              ON A.NRSEQVENDA = B.NRSEQVENDA
             AND A.CDFILIAL   = B.CDFILIAL
             AND A.CDCAIXA    = B.CDCAIXA";

	const GET_NRTRANSACOES =
		"SELECT COUNT(DISTINCT(V.NRSEQVENDA))  AS NRTRANSACOES
		   FROM VENDA V
		  WHERE (V.CDFILIAL      = :CDFILIAL   )
		    AND (V.CDCAIXA       = :CDCAIXA    )
		    AND (CONVERT(VARCHAR,V.DTENTRVENDA,103) = CONVERT(VARCHAR,:DTABERTUR,103) )
		    AND (V.IDSITUVENDA   = 'O')";

    const GET_NRPRIMSEQ =
    	"SELECT MIN(NRNOTAFISCALCE) AS NRPRIMSEQ
    	   FROM VENDA
    	  WHERE CDFILIAL = :CDFILIAL
    	    AND CDCAIXA  = :CDCAIXA
    	    AND CONVERT(VARCHAR,DTENTRVENDA ,103) = CONVERT(VARCHAR,:DTABERCAIX,103)";


	const GET_NRFINALSEQ =
	    "SELECT MAX(NRNOTAFISCALCE) AS NRFINALSEQ
           FROM VENDA
          WHERE CDFILIAL = :CDFILIAL
            AND CDCAIXA  = :CDCAIXA
            AND CONVERT(VARCHAR,DTENTRVENDA ,103) = CONVERT(VARCHAR,:DTABERCAIX,103)";

	const GET_CANCELAMENTOS =
	    "SELECT COUNT(DISTINCT(V.NRSEQVENDA)) AS TRANSCANC,
	             ISNULL(SUM(ROUND(I.QTPRODVEND*(I.VRUNITVEND+I.VRUNITVENDCL)+ISNULL(I.VRACRITVEND,0)- ISNULL(I.VRDESITVEND,0),2,1)),0) AS VRCANCEL
	       FROM ITEMVENDA I JOIN VENDA V ON I.CDFILIAL   = V.CDFILIAL
	                                    AND I.CDCAIXA    = V.CDCAIXA
	                                    AND I.NRSEQVENDA = V.NRSEQVENDA
	      WHERE V.CDFILIAL    = :CDFILIAL
	        AND V.CDCAIXA     = :CDCAIXA
	        AND CONVERT(VARCHAR,V.DTABERTUR,103) = CONVERT(VARCHAR,:DTABERTUR,103)
	        AND V.IDSITUVENDA = 'C'";

	const GET_VRACRITVENDFIN =
	    "SELECT SUM(ISNULL(I.VRACRITVEND,0)) AS VRACRITVENDFIN
	       FROM ITEMVENDA I JOIN VENDA V ON I.CDFILIAL   = V.CDFILIAL
	                                    AND I.CDCAIXA    = V.CDCAIXA
	                                    AND I.NRSEQVENDA = V.NRSEQVENDA
	      WHERE V.CDFILIAL    = :CDFILIAL
	        AND V.CDCAIXA     = :CDCAIXA
	        AND CONVERT(VARCHAR,V.DTABERTUR,103)   = CONVERT(VARCHAR,:DTABERTUR,103)
	        AND V.IDSITUVENDA = 'O'";

	const GET_VRDESITVENDFIN =
	    "SELECT SUM(ISNULL(I.VRDESITVEND,0)) AS VRDESITVENDFIN
	       FROM ITEMVENDA I JOIN VENDA V ON I.CDFILIAL   = V.CDFILIAL
	                                    AND I.CDCAIXA    = V.CDCAIXA
	                                    AND I.NRSEQVENDA = V.NRSEQVENDA
	      WHERE V.CDFILIAL    = :CDFILIAL
	        AND V.CDCAIXA     = :CDCAIXA
	        AND CONVERT(VARCHAR,V.DTABERTUR,103) = CONVERT(VARCHAR,:DTABERTUR,103)
	        AND V.IDSITUVENDA = 'O'";

	const GET_VRACRITVEND =
		"SELECT SUM(ISNULL(I.VRACRITVEND,0)) AS VRACRITVEND
		   FROM ITEMVENDA I JOIN VENDA V ON I.CDFILIAL   = V.CDFILIAL
		                                    AND I.CDCAIXA    = V.CDCAIXA
		                                    AND I.NRSEQVENDA = V.NRSEQVENDA
		  WHERE V.CDFILIAL    = :CDFILIAL
		    AND V.CDCAIXA     = :CDCAIXA
		    AND V.DTABERTUR   = :DTABERTUR
		    AND V.IDSITUVENDA = 'O'";

    const GET_VRDESITVEND =
		"SELECT SUM(ISNULL(I.VRDESITVEND,0)) AS VRDESITVEND
  		   FROM ITEMVENDA I JOIN VENDA V ON I.CDFILIAL   = V.CDFILIAL
		                                    AND I.CDCAIXA    = V.CDCAIXA
		                                    AND I.NRSEQVENDA = V.NRSEQVENDA
          WHERE V.CDFILIAL = :CDFILIAL
            AND V.CDCAIXA     = :CDCAIXA
            AND CONVERT(VARCHAR,V.DTABERTUR,103) = CONVERT(VARCHAR,:DTABERTUR,103)
            AND V.IDSITUVENDA = 'O'";

    const GET_VRMOVIVENDFIN =
        "SELECT SUM(VRMOVIVEND) AS VRMOVIVENDFIN
           FROM MOVCAIXA
          WHERE CDFILIAL     = :CDFILIAL
            AND CDCAIXA      = :CDCAIXA
            AND CONVERT(VARCHAR,DTABERCAIX,103) = CONVERT(VARCHAR,:DTABERCAIX,103)
            AND IDTIPOMOVIVE = 'A'";

    const GET_VRMOVIVEND =
        "SELECT VRMOVIVEND
           FROM MOVCAIXA
          WHERE CDFILIAL     = :CDFILIAL
            AND CDCAIXA      = :CDCAIXA
            AND CONVERT(VARCHAR,DTABERCAIX,103) = CONVERT(VARCHAR,:DTABERCAIX,103)
            AND IDTIPOMOVIVE = 'A'";

	const GET_IMPOSTOS =
        "SELECT SGIMPOSTO, P.VRPEALPRODIT,
        		SUM((QTPRODVEND*VRUNITVEND)-ISNULL(VRDESITVEND,0)+ISNULL(VRACRITVEND,0)) AS VRBASE,
        		SUM(((QTPRODVEND*VRUNITVEND)-ISNULL(VRDESITVEND,0)+ISNULL(VRACRITVEND,0))/100*P.VRPEALPRODIT) AS VRIMPOSTO
		   FROM ITEMVENDA I JOIN VENDA V ON I.CDFILIAL   = V.CDFILIAL
                                              AND I.CDCAIXA    = V.CDCAIXA
                                              AND I.NRSEQVENDA = V.NRSEQVENDA
                                             JOIN ITVENDAIMPOS P
                                               ON I.CDFILIAL     = P.CDFILIAL
                                              AND I.CDCAIXA      = P.CDCAIXA
                                              AND I.NRSEQVENDA   = P.NRSEQVENDA
                                              AND I.NRSEQUITVEND = P.NRSEQUITVEND
                                             JOIN IMPOSTO IM
                                               ON P.CDIMPOSTO = IM.CDIMPOSTO
          WHERE I.CDFILIAL = :CDFILIAL
            AND I.CDCAIXA  = :CDCAIXA
            AND CONVERT(VARCHAR,V.DTENTRVENDA, 103) = CONVERT(VARCHAR,:DTABERTUR,103)
            AND V.IDSITUVENDA = 'O'
       GROUP BY SGIMPOSTO, P.VRPEALPRODIT";

	const GET_PAGAMENTOSFIN =
	      "SELECT RC.CDTIPORECE, RC.NMTIPORECE, RC.VRMOVIVEND AS VALOR_TOTAL, RC.VRTROCO
	     FROM (SELECT T.CDTIPORECE, T.NMTIPORECE,
	                      SUM(ISNULL(S.VRMOVIVEND,0)) AS VRTROCO,
	                      SUM(ISNULL(E.VRMOVIVEND,0) - ISNULL(S.VRMOVIVEND,0)) AS VRMOVIVEND
	                 FROM TIPORECE T
	                 JOIN (SELECT CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQVENDA,
	                                         CDTIPORECE, SUM(ISNULL(VRMOVIVEND,0)) AS VRMOVIVEND
	                                    FROM MOVCAIXA
	                                   WHERE CDFILIAL  = :CDFILIAL
	                                     AND CDCAIXA   = :CDCAIXA
	                                     AND CONVERT(VARCHAR,DTABERCAIX,103) = CONVERT(VARCHAR, :DTABERCAIX ,103)
	                                     AND IDTIPOMOVIVE IN ('E','T')
	                                   GROUP BY CDFILIAL, CDCAIXA, DTABERCAIX, CDTIPORECE, NRSEQVENDA) E
	                   ON T.CDTIPORECE  = E.CDTIPORECE
	            LEFT JOIN
	                      (SELECT CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQVENDA,
	                              CDTIPORECE, SUM(ISNULL(VRMOVIVEND,0)) AS VRMOVIVEND
	                         FROM MOVCAIXA
	                        WHERE CDFILIAL = :CDFILIAL
	                          AND CDCAIXA  = :CDCAIXA
	                          AND CONVERT(VARCHAR,DTABERCAIX,103) = CONVERT(VARCHAR, :DTABERCAIX ,103)
	                          AND IDTIPOMOVIVE = 'S'
	                     GROUP BY CDFILIAL, CDCAIXA, DTABERCAIX, CDTIPORECE, NRSEQVENDA) S
	                   ON E.CDTIPORECE = S.CDTIPORECE
	                  AND E.NRSEQVENDA = S.NRSEQVENDA
	    WHERE E.CDFILIAL    = :CDFILIAL
	      AND E.CDCAIXA     = :CDCAIXA
	 GROUP BY T.CDTIPORECE, T.NMTIPORECE) RC";

	const GET_PAGAMENTOS =
	      "SELECT RC.CDTIPORECE, RC.NMTIPORECE, RC.VRMOVIVEND AS VALOR_TOTAL, RC.VRTROCO
	     FROM (SELECT T.CDTIPORECE, T.NMTIPORECE,
	                      SUM(ISNULL(S.VRMOVIVEND,0)) AS VRTROCO,
	                      SUM(ISNULL(E.VRMOVIVEND,0) - ISNULL(S.VRMOVIVEND,0)) AS VRMOVIVEND
	                 FROM TIPORECE T
	                 JOIN (SELECT CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQVENDA,
	                                         CDTIPORECE, SUM(ISNULL(VRMOVIVEND,0)) AS VRMOVIVEND
	                                    FROM MOVCAIXA
	                                   WHERE CDFILIAL  = :CDFILIAL
	                                     AND CDCAIXA   = :CDCAIXA
	                                     AND CONVERT(VARCHAR,DTABERCAIX,103) = CONVERT(VARCHAR, :DTABERCAIX ,103)
	                                     AND IDTIPOMOVIVE IN ('E','T')
	                                   GROUP BY CDFILIAL, CDCAIXA, DTABERCAIX, CDTIPORECE, NRSEQVENDA) E
	                   ON T.CDTIPORECE  = E.CDTIPORECE
	            LEFT JOIN
	                      (SELECT CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQVENDA,
	                              CDTIPORECE, SUM(ISNULL(VRMOVIVEND,0)) AS VRMOVIVEND
	                         FROM MOVCAIXA
	                        WHERE CDFILIAL = :CDFILIAL
	                          AND CDCAIXA  = :CDCAIXA
	                          AND CONVERT(VARCHAR,DTABERCAIX,103) = CONVERT(VARCHAR, :DTABERCAIX ,103)
	                          AND IDTIPOMOVIVE = 'S'
	                     GROUP BY CDFILIAL, CDCAIXA, DTABERCAIX, CDTIPORECE, NRSEQVENDA) S
	                   ON E.CDTIPORECE = S.CDTIPORECE
	                  AND E.NRSEQVENDA = S.NRSEQVENDA
	    WHERE E.CDFILIAL    = :CDFILIAL
	      AND E.CDCAIXA     = :CDCAIXA
	 GROUP BY T.CDTIPORECE, T.NMTIPORECE) RC";

}