<?php

namespace Zeedhi\PaymentPlatform\Implementations\SoftwareExpress;

use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyTransaction;
use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction;
use Zeedhi\PaymentPlatform\CreditCard;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\Exception;

class PaymentImplTest extends \PHPUnit_Framework_TestCase {

	protected $client;
	protected $paymentImplSoftwareExpress;
	
	public function setUp(){
		$this->client = $this->getMockBuilder(SoftwareExpressRequireFiles::class)
							 ->setMethods(array('clientFault', 'clientGetError', 'beginTransaction', 'doPayment', 'store', 'doHashPayment', 'doPaymentResponse', 'storeResponse'))
                             ->disableOriginalConstructor()
                             ->getMock();

		$this->paymentImplSoftwareExpress = new PaymentImpl("000000001", "00000001", $this->client);
	}

	public function testWebServiceUnavaliable(){

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Software Express está indisponível.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");
		$customer = new Customer;
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer, $customer);

        $this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(true);

		$this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testWebServiceConnectionFail(){

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Conexão com Software Express falhou.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

        $this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(true);

		$this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testNonInstantBuyTransactionVisa(){

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}

	public function testNonInstantBuyTransactionMastercard(){
		
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Mastercard");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}

	public function testNonInstantBuyTransactionAmex(){
		
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Amex");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}

	public function testNonInstantBuyTransactionHipercard(){
		
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Hipercard");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}

	public function testNonInstantBuyTransactionAura(){
		
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}

	public function testNonInstantBuyTransactionDinners(){
		
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Diners");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}

	public function testNonInstantBuyTransactionElo(){
		
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}
	
	public function testNonInstantBuyTransactionAlelo(){
		
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Alelo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}
	
	public function testNonInstantBuyTransactionSodexo(){
		
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Sodexo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}

	public function testNonInstantBuyTransactionunsupportedFlag(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Unsupported flag.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "unsupportedFlag");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        
		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testBeginTransactionExceptionCode1(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Requisição nula ou vazia.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 1, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testBeginTransactionExceptionCode3(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Campo amount inválido ou ultrapassa o tamanho limite.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 3, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testBeginTransactionExceptionCode4(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Campo merchantUSN inválido.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 4, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testBeginTransactionExceptionCode5(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Campo merchantId nulo ou vazio.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 5, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testBeginTransactionExceptionCode131(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Campo amount nulo ou negativo.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 131, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testBeginTransactionExceptionCode132(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Campo merchantUSN ultrapassa o limite de tamanho.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 132, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testBeginTransactionExceptionCode133(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Campo orderId ultrapassa o limite de tamanho.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 133, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testBeginTransactionExceptionCode151(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Campo merchantId inválido.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 151, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testBeginTransactionExceptionCode153(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Loja está inativa no e-SiTef.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 153, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testBeginTransactionExceptionCode1000(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Erro inesperado no e-SiTef. Favor entrar em contato com o suporte.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 1000, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testBeginTransactionExceptionDefault(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Ocorreu um erro na API Software Express");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Elo");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 123456, "nit" => "00000000123"));

		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);
		
		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);

	}

	public function testDoPaymentExceptionResponseINV(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Transação não foi criada com sucesso, algum parâmetro foi enviado incorretamente para iniciar a transação.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "INV"));

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testDoPaymentExceptionResponseEXP(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Transação expirou");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "EXP"));

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testDoPaymentExceptionResponseCAN(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Transação foi cancelada pelo usuário.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "CAN"));

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testDoPaymentExceptionResponseABA(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Transação foi abandonada pelo usuário.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "ABA"));

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testDoPaymentExceptionResponseNEG(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Pagamento negado pela Instituição Financeira.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "NEG"));

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testDoPaymentExceptionResponseEST(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Pagamento cancelado pelo BackOffice");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "EST"));

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testDoPaymentExceptionResponseTNE(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Transação não encontrada no sistema. Código NIT incorreto.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "TNE"));

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testDoPaymentExceptionResponseBLQ(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Transação bloqueada por excesso de tentativas.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "BLQ"));

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testDoPaymentExceptionResponseERR(){
		

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Erro de comunicação com a autorizadora. Tente novamente.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "ERR"));

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testDoPaymentExceptionResponseDefault(){
		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Ocorreu um erro na API Software Express: Erro inesperado.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Aura");
		$customer = new Customer;
		$creditCardTrasnaction = new NonInstantBuyTransaction("100.00", "0000001", $creditCard, $customer);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("paymentResponse" => array("transactionStatus" => "ERRRR"));

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->nonInstantBuyTransaction($creditCardTrasnaction);
	}

	public function testFactoryWithFlagSandBox(){

		$factory = $this->paymentImplSoftwareExpress->factory("000000001", "000000002", PaymentImpl::SANDBOX);
		$this->assertInstanceOf(PaymentImpl::class, $factory);
	}

	public function testFactoryWithFlagProduction(){

		$factory = $this->paymentImplSoftwareExpress->factory("000000001", "000000002", PaymentImpl::PRODUCTION);
		$this->assertInstanceOf(PaymentImpl::class, $factory);
	}

	public function testFactoryWithFlagException(){
		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Unsupported Factory Flag.");
		$this->expectExceptionCode(5);
		$factory = $this->paymentImplSoftwareExpress->factory("000000001", "000000002", "error");
	}

	public function testCreateToken() {
		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");
		$customer = new Customer;
		$customer->setId('123');

		$invalidChars = array('20', '/', ':', ' ');
        $merchantUSN = str_replace($invalidChars, '', date('Y/m/d H:i:s'));

		$storeResponse = array("storeResponse" => array("status" => "CON", 
														"cardHash" => "asdoiu123jasd", 
														"CustomerId" => $merchantUSN));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('store')
        			 ->willReturn($storeResponse);

        $this->client->expects($this->once())
        			 ->method('storeResponse')
        			 ->willReturn($storeResponse);

		$createTokenResponse = $this->paymentImplSoftwareExpress->createToken($creditCard, $customer);

		$this->assertEquals($createTokenResponse["InstantBuyKey"], $storeResponse["storeResponse"]["cardHash"]);
		$this->assertEquals($createTokenResponse["CustomerId"], $storeResponse["storeResponse"]["CustomerId"]);
	}

	public function testCheckCreateTokenErrorsINV() {
		$this->setExpectedException(Exception::class);
		$this->expectExceptionCode(Exception::CREATE_TOKEN_ERROR);
		$this->expectExceptionMessage("Ocorreu um erro durante o cadastro do cartão: Transação não foi criada com sucesso, algum parâmetro foi enviado incorretamente para iniciar a transação.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");
		$customer = new Customer;
		$customer->setId('123');

		$storeResponse = array("storeResponse" => array("status" => "INV"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('store')
        			 ->willReturn($storeResponse);

        $this->client->expects($this->once())
        			 ->method('storeResponse')
        			 ->willReturn($storeResponse);

		$this->paymentImplSoftwareExpress->createToken($creditCard, $customer);
	}

	public function testCheckCreateTokenErrorsEXP() {
		$this->setExpectedException(Exception::class);
		$this->expectExceptionCode(Exception::CREATE_TOKEN_ERROR);
		$this->expectExceptionMessage("Ocorreu um erro durante o cadastro do cartão: Armazenamento expirou devido a tempo excessivo sem a tualização por parte da  loja (sem conexão com o usuário/comprador).");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");
		$customer = new Customer;
		$customer->setId('123');

		$storeResponse = array("storeResponse" => array("status" => "EXP"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('store')
        			 ->willReturn($storeResponse);

        $this->client->expects($this->once())
        			 ->method('storeResponse')
        			 ->willReturn($storeResponse);

		$this->paymentImplSoftwareExpress->createToken($creditCard, $customer);

	}

	public function testCheckCreateTokenErrorsNEG() {
		$this->setExpectedException(Exception::class);
		$this->expectExceptionCode(Exception::CREATE_TOKEN_ERROR);
		$this->expectExceptionMessage("Ocorreu um erro durante o cadastro do cartão: Armazenamento negado. Cartão inválido.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");
		$customer = new Customer;
		$customer->setId('123');

		$storeResponse = array("storeResponse" => array("status" => "NEG"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('store')
        			 ->willReturn($storeResponse);

        $this->client->expects($this->once())
        			 ->method('storeResponse')
        			 ->willReturn($storeResponse);

		$this->paymentImplSoftwareExpress->createToken($creditCard, $customer);

	}

	public function testCheckCreateTokenErrorsCAN() {
		$this->setExpectedException(Exception::class);
		$this->expectExceptionCode(Exception::CREATE_TOKEN_ERROR);
		$this->expectExceptionMessage("Ocorreu um erro durante o cadastro do cartão: Armazenamento do cartão cancelado pelo cliente. Dados removidos do e-SiTef a pedidos da loja.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");
		$customer = new Customer;
		$customer->setId('123');

		$storeResponse = array("storeResponse" => array("status" => "CAN"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('store')
        			 ->willReturn($storeResponse);

        $this->client->expects($this->once())
        			 ->method('storeResponse')
        			 ->willReturn($storeResponse);

		$this->paymentImplSoftwareExpress->createToken($creditCard, $customer);

	}

	public function testCheckCreateTokenErrorsUnexpectedError() {
		$this->setExpectedException(Exception::class);
		$this->expectExceptionCode(Exception::API_SOFTWARE_EXPRESS_ERROR);
		$this->expectExceptionMessage("Ocorreu um erro na API Software Express: Erro inesperado.");

		$creditCard = new CreditCard("TEKNISA TESTES", "4000000000000001", "03", "20", "000", "Visa");
		$customer = new Customer;
		$customer->setId('123');

		$storeResponse = array("storeResponse" => array("status" => "ERROR"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('store')
        			 ->willReturn($storeResponse);

        $this->client->expects($this->once())
        			 ->method('storeResponse')
        			 ->willReturn($storeResponse);

		$this->paymentImplSoftwareExpress->createToken($creditCard, $customer);

	}

	public function testInstantBuyTransactionVisa(){

		$creditCardToken = "IAIDNusa8d7a8dada8sd7as7d==";
		$customer = new Customer;
		$customer->setId('123');

		$creditCardTrasnaction = new InstantBuyTransaction("100.00", "0000001", $creditCardToken, $customer);
		$creditCardTrasnaction->setFlag(CreditCard::FLAG_VISA);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("recurrentPaymentResponse" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doHashPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->instantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}

	public function testcheckPaymentErrorsUnexpectedError(){
		$this->setExpectedException(Exception::class);
		$this->expectExceptionCode(Exception::API_SOFTWARE_EXPRESS_ERROR);
		$this->expectExceptionMessage("Ocorreu um erro na API Software Express: Erro inesperado.");

		$creditCardToken = "IAIDNusa8d7a8dada8sd7as7d==";
		$customer = new Customer;
		$customer->setId('123');

		$creditCardTrasnaction = new InstantBuyTransaction("100.00", "0000001", $creditCardToken, $customer);
		$creditCardTrasnaction->setFlag(CreditCard::FLAG_VISA);

		$responseBeginTransaction = array("transactionResponse" => array("responseCode" => 0, "nit" => "00000000123"));

		$responseDoPayment = array("error" => array("transactionStatus" => "CON"));
		
		$this->client->expects($this->once())
        			 ->method('clientFault')
        			 ->willReturn(false);

        $this->client->expects($this->once())
        			 ->method('clientGetError')
        			 ->willReturn(false);

		$this->client->expects($this->once())
        			 ->method('beginTransaction')
        			 ->willReturn($responseBeginTransaction);

        $this->client->expects($this->once())
        			 ->method('doHashPayment')
        			 ->willReturn($responseDoPayment);

        $this->client->expects($this->once())
        			 ->method('doPaymentResponse')
        			 ->willReturn($responseDoPayment);

		$nit = $this->paymentImplSoftwareExpress->instantBuyTransaction($creditCardTrasnaction);

		$this->assertEquals($nit, $responseBeginTransaction["transactionResponse"]["nit"]);
	}
}
