<?php

namespace Zeedhi\PaymentPlatform\Implementations\BoletoPHP;

use Zeedhi\PaymentPlatform\BankSlip\BankSlipTransaction;
use Zeedhi\PaymentPlatform\BankSlip\Store;
use Zeedhi\PaymentPlatform\BankSlip\StoreBankAccount;

use Zeedhi\PaymentPlatform\Exception;
use Zeedhi\PaymentPlatform\Address;
use Zeedhi\PaymentPlatform\Customer;

class PaymentImplTest extends \PHPUnit_Framework_TestCase {

	protected $bankSlip;
	protected $requireFiles;
	protected $dateBankSlip;
	protected $dateBankSlipPlusFiveDays;

	public function setUp(){
		
		$storeAddress = new Address("Rua Endereço", "111", "Cj 100", "Bairro", "BH", "MG", "41651-65", "BR");
		$storeBankInfo = new StoreBankAccount("5689", "1", "06511", "7", "121456", "1234567");
		$storeInfo = new Store("Nome Fantasia", $storeAddress, $storeBankInfo, "www.nomefantasia.com", "65146516", "Nome registro", "123456", "123456");

		$this->requireFiles = $this->getMockBuilder(BankSlipRequireFiles::class)
							 ->setMethods(array('bankSlipConstruct'))
                             ->disableOriginalConstructor()
                             ->getMock();

		$this->bankSlip = new PaymentImpl($storeInfo, $this->requireFiles);

		$this->dateBankSlip = date("d/m/Y");
		$this->dateBankSlipPlusFiveDays = date("d/m/Y", strtotime("+5 days"));
	}

	public function testBoletoTransactionItau(){

		$clientAddress = new Address("Rua Pernambuco", "1000", "Cj 102", "Funcionários", "Belo Horizonte", "Minas Gerais", "30130-151", "BR");
		$clientInfo = new Customer;
		$clientInfo->setName("Teste Teknisa");
		$clientInfo->setAddress($clientAddress);
		$clientInfo->setDocument("12985019999");
		$clientInfo->setEmail("lucas.lacerda@teknisa.com");
		$clientInfo->setPhone("(31)991664747");
		$bankSlip = new BankSlipTransaction("341", "100.25", "123123123", "654987", "Boleto de testes Teknisa", "5", $clientInfo);

		$this->requireFiles->expects($this->once())
        			 ->method('bankSlipConstruct')
        			 ->with(
        			 	array(
	        			 	'carteira' => '109' ,
	        			 	'bankName' => 'itau' ,
	        			 	'nosso_numero' => '121456' ,
	        			 	'numero_documento' => '123123123' ,
	        			 	'data_vencimento' => $this->dateBankSlipPlusFiveDays,
	        			 	'data_documento' => $this->dateBankSlip ,
	        			 	'data_processamento' => $this->dateBankSlip ,
	        			 	'valor_boleto' => '100,25' ,
	        			 	'sacado' => 'Teste Teknisa' ,
	        			 	'endereco1' => 'Rua Pernambuco 1000 Cj 102 Funcionários Belo Horizonte Minas Gerais 30130-151 BR' ,
	        			 	'endereco2' => '' ,
	        			 	'demonstrativo1' => 'Boleto de testes Teknisa' ,
	        			 	'demonstrativo2' => '' ,
	        			 	'demonstrativo3' => '' ,
	        			 	'instrucoes1' => '- Favor nao receber apos o vencimento' ,
	        			 	'instrucoes2' => 'www.nomefantasia.com' ,
	        			 	'instrucoes3' => '' ,
	        			 	'instrucoes4' => '' ,
	        			 	'quantidade' => '001' ,
	        			 	'valor_unitario' => '100,25' ,
	        			 	'aceite' => '' ,
	        			 	'especie' => 'R$' ,
	        			 	'especie_doc' => 'DS' ,
	        			 	'agencia' => '5689' ,
	        			 	'agencia_dv' => '1' ,
	        			 	'conta' => '06511' ,
	        			 	'conta_dv' => '7' ,
	        			 	'conta_cedente' => '' ,
	        			 	'conta_cedente_dv' => '' ,
	        			 	'convenio' => '123456' ,
	        			 	'contrato' => '123456' ,
	        			 	'formatacao_convenio' => 6 ,
	        			 	'formatacao_nosso_numero' => '2' ,
	        			 	'identificacao' => 'Nome Fantasia' ,
	        			 	'cpf_cnpj' => '65146516' ,
	        			 	'endereco' => 'Rua Endereço 111 Cj 100 Bairro BH MG 41651-65 BR' ,
	        			 	'cidade_uf' => 'BH MG' ,
	        			 	'cedente' => 'Nome registro'
 						)
 					);

        $instantBuyKey = $this->bankSlip->bankSlipTransaction($bankSlip);
	}

	public function testBoletoTransactionBradesco(){

		$clientAddress = new Address("Rua Pernambuco", "1000", "Cj 102", "Funcionários", "Belo Horizonte", "Minas Gerais", "30130-151", "BR");
		$clientInfo = new Customer;
		$clientInfo->setName("Teste Teknisa");
		$clientInfo->setAddress($clientAddress);
		$clientInfo->setDocument("12985019999");
		$clientInfo->setEmail("lucas.lacerda@teknisa.com");
		$clientInfo->setPhone("(31)991664747");
		$bankSlip = new BankSlipTransaction("237", "100.25", "123123123", "654987", "Boleto de testes Teknisa", "5", $clientInfo);

		$this->requireFiles->expects($this->once())
        			 ->method('bankSlipConstruct')
        			 ->with(
        			 	array(
	        			 	'carteira' => '06' ,
	        			 	'bankName' => 'bradesco' ,
	        			 	'nosso_numero' => '121456' ,
	        			 	'numero_documento' => '123123123' ,
	        			 	'data_vencimento' => $this->dateBankSlipPlusFiveDays,
	        			 	'data_documento' => $this->dateBankSlip ,
	        			 	'data_processamento' => $this->dateBankSlip ,
	        			 	'valor_boleto' => '100,25' ,
	        			 	'sacado' => 'Teste Teknisa' ,
	        			 	'endereco1' => 'Rua Pernambuco 1000 Cj 102 Funcionários Belo Horizonte Minas Gerais 30130-151 BR' ,
	        			 	'endereco2' => '' ,
	        			 	'demonstrativo1' => 'Boleto de testes Teknisa' ,
	        			 	'demonstrativo2' => '' ,
	        			 	'demonstrativo3' => '' ,
	        			 	'instrucoes1' => '- Favor nao receber apos o vencimento' ,
	        			 	'instrucoes2' => 'www.nomefantasia.com' ,
	        			 	'instrucoes3' => '' ,
	        			 	'instrucoes4' => '' ,
	        			 	'quantidade' => '001' ,
	        			 	'valor_unitario' => '100,25' ,
	        			 	'aceite' => '' ,
	        			 	'especie' => 'R$' ,
	        			 	'especie_doc' => 'DS' ,
	        			 	'agencia' => '5689' ,
	        			 	'agencia_dv' => '1' ,
	        			 	'conta' => '06511' ,
	        			 	'conta_dv' => '7' ,
	        			 	'conta_cedente' => '' ,
	        			 	'conta_cedente_dv' => '' ,
	        			 	'convenio' => '123456' ,
	        			 	'contrato' => '123456' ,
	        			 	'formatacao_convenio' => 6 ,
	        			 	'formatacao_nosso_numero' => '2' ,
	        			 	'identificacao' => 'Nome Fantasia' ,
	        			 	'cpf_cnpj' => '65146516' ,
	        			 	'endereco' => 'Rua Endereço 111 Cj 100 Bairro BH MG 41651-65 BR' ,
	        			 	'cidade_uf' => 'BH MG' ,
	        			 	'cedente' => 'Nome registro',
	        			 	'codigo_cedente' => '1234567',
							'codigo_cedente_dv' => false
 						)
 					);

        $instantBuyKey = $this->bankSlip->bankSlipTransaction($bankSlip);
	}	

	public function testBoletoTransactionBancoBrasil(){

		$clientAddress = new Address("Rua Pernambuco", "1000", "Cj 102", "Funcionários", "Belo Horizonte", "Minas Gerais", "30130-151", "BR");
		$clientInfo = new Customer;
		$clientInfo->setName("Teste Teknisa");
		$clientInfo->setAddress($clientAddress);
		$clientInfo->setDocument("12985019999");
		$clientInfo->setEmail("lucas.lacerda@teknisa.com");
		$clientInfo->setPhone("(31)991664747");
		$bankSlip = new BankSlipTransaction("001", "100.25", "123123123", "654987", "Boleto de testes Teknisa", "5", $clientInfo);

		$this->requireFiles->expects($this->once())
        			 ->method('bankSlipConstruct')
        			 ->with(
        			 	array(
	        			 	'carteira' => '17' ,
	        			 	'bankName' => 'bb' ,
	        			 	'nosso_numero' => '121456' ,
	        			 	'numero_documento' => '123123123' ,
	        			 	'data_vencimento' => $this->dateBankSlipPlusFiveDays,
	        			 	'data_documento' => $this->dateBankSlip ,
	        			 	'data_processamento' => $this->dateBankSlip ,
	        			 	'valor_boleto' => '100,25' ,
	        			 	'sacado' => 'Teste Teknisa' ,
	        			 	'endereco1' => 'Rua Pernambuco 1000 Cj 102 Funcionários Belo Horizonte Minas Gerais 30130-151 BR' ,
	        			 	'endereco2' => '' ,
	        			 	'demonstrativo1' => 'Boleto de testes Teknisa' ,
	        			 	'demonstrativo2' => '' ,
	        			 	'demonstrativo3' => '' ,
	        			 	'instrucoes1' => '- Favor nao receber apos o vencimento' ,
	        			 	'instrucoes2' => 'www.nomefantasia.com' ,
	        			 	'instrucoes3' => '' ,
	        			 	'instrucoes4' => '' ,
	        			 	'quantidade' => '001' ,
	        			 	'valor_unitario' => '100,25' ,
	        			 	'aceite' => '' ,
	        			 	'especie' => 'R$' ,
	        			 	'especie_doc' => 'DS' ,
	        			 	'agencia' => '5689' ,
	        			 	'agencia_dv' => '1' ,
	        			 	'conta' => '06511' ,
	        			 	'conta_dv' => '7' ,
	        			 	'conta_cedente' => '' ,
	        			 	'conta_cedente_dv' => '' ,
	        			 	'convenio' => '123456' ,
	        			 	'contrato' => '123456' ,
	        			 	'formatacao_convenio' => 6 ,
	        			 	'formatacao_nosso_numero' => '2' ,
	        			 	'identificacao' => 'Nome Fantasia' ,
	        			 	'cpf_cnpj' => '65146516' ,
	        			 	'endereco' => 'Rua Endereço 111 Cj 100 Bairro BH MG 41651-65 BR' ,
	        			 	'cidade_uf' => 'BH MG' ,
	        			 	'cedente' => 'Nome registro'
 						)
 					);

        $instantBuyKey = $this->bankSlip->bankSlipTransaction($bankSlip);
	}	

	public function testBoletoTransactionHsbc(){

		$clientAddress = new Address("Rua Pernambuco", "1000", "Cj 102", "Funcionários", "Belo Horizonte", "Minas Gerais", "30130-151", "BR");
		$clientInfo = new Customer;
		$clientInfo->setName("Teste Teknisa");
		$clientInfo->setAddress($clientAddress);
		$clientInfo->setDocument("12985019999");
		$clientInfo->setEmail("lucas.lacerda@teknisa.com");
		$clientInfo->setPhone("(31)991664747");
		$bankSlip = new BankSlipTransaction("399", "100.25", "123123123", "654987", "Boleto de testes Teknisa", "5", $clientInfo);

		$this->requireFiles->expects($this->once())
        			 ->method('bankSlipConstruct')
        			 ->with(
        			 	array(
	        			 	'carteira' => 'CNR' ,
	        			 	'bankName' => 'hsbc' ,
	        			 	'nosso_numero' => '121456' ,
	        			 	'numero_documento' => '123123123' ,
	        			 	'data_vencimento' => $this->dateBankSlipPlusFiveDays,
	        			 	'data_documento' => $this->dateBankSlip ,
	        			 	'data_processamento' => $this->dateBankSlip ,
	        			 	'valor_boleto' => '100,25' ,
	        			 	'sacado' => 'Teste Teknisa' ,
	        			 	'endereco1' => 'Rua Pernambuco 1000 Cj 102 Funcionários Belo Horizonte Minas Gerais 30130-151 BR' ,
	        			 	'endereco2' => '' ,
	        			 	'demonstrativo1' => 'Boleto de testes Teknisa' ,
	        			 	'demonstrativo2' => '' ,
	        			 	'demonstrativo3' => '' ,
	        			 	'instrucoes1' => '- Favor nao receber apos o vencimento' ,
	        			 	'instrucoes2' => 'www.nomefantasia.com' ,
	        			 	'instrucoes3' => '' ,
	        			 	'instrucoes4' => '' ,
	        			 	'quantidade' => '001' ,
	        			 	'valor_unitario' => '100,25' ,
	        			 	'aceite' => '' ,
	        			 	'especie' => 'R$' ,
	        			 	'especie_doc' => 'DS' ,
	        			 	'agencia' => '5689' ,
	        			 	'agencia_dv' => '1' ,
	        			 	'conta' => '06511' ,
	        			 	'conta_dv' => '7' ,
	        			 	'conta_cedente' => '' ,
	        			 	'conta_cedente_dv' => '' ,
	        			 	'convenio' => '123456' ,
	        			 	'contrato' => '123456' ,
	        			 	'formatacao_convenio' => 6 ,
	        			 	'formatacao_nosso_numero' => '2' ,
	        			 	'identificacao' => 'Nome Fantasia' ,
	        			 	'cpf_cnpj' => '65146516' ,
	        			 	'endereco' => 'Rua Endereço 111 Cj 100 Bairro BH MG 41651-65 BR' ,
	        			 	'cidade_uf' => 'BH MG' ,
	        			 	'cedente' => 'Nome registro',
	        			 	'codigo_cedente' =>	'1234567'
 						)
 					);

        $instantBuyKey = $this->bankSlip->bankSlipTransaction($bankSlip);
	}

	public function testBoletoTransactionSantander(){

		$clientAddress = new Address("Rua Pernambuco", "1000", "Cj 102", "Funcionários", "Belo Horizonte", "Minas Gerais", "30130-151", "BR");
		$clientInfo = new Customer;
		$clientInfo->setName("Teste Teknisa");
		$clientInfo->setAddress($clientAddress);
		$clientInfo->setDocument("12985019999");
		$clientInfo->setEmail("lucas.lacerda@teknisa.com");
		$clientInfo->setPhone("(31)991664747");
		$bankSlip = new BankSlipTransaction("033", "100.25", "123123123", "654987", "Boleto de testes Teknisa", "5", $clientInfo);

		$this->requireFiles->expects($this->once())
        			 ->method('bankSlipConstruct')
        			 ->with(
        			 	array(
	        			 	'carteira' => '102' ,
	        			 	'bankName' => 'santander' ,
	        			 	'nosso_numero' => '121456' ,
	        			 	'numero_documento' => '123123123' ,
	        			 	'data_vencimento' => $this->dateBankSlipPlusFiveDays,
	        			 	'data_documento' => $this->dateBankSlip ,
	        			 	'data_processamento' => $this->dateBankSlip ,
	        			 	'valor_boleto' => '100,25' ,
	        			 	'sacado' => 'Teste Teknisa' ,
	        			 	'endereco1' => 'Rua Pernambuco 1000 Cj 102 Funcionários Belo Horizonte Minas Gerais 30130-151 BR' ,
	        			 	'endereco2' => '' ,
	        			 	'demonstrativo1' => 'Boleto de testes Teknisa' ,
	        			 	'demonstrativo2' => '' ,
	        			 	'demonstrativo3' => '' ,
	        			 	'instrucoes1' => '- Favor nao receber apos o vencimento' ,
	        			 	'instrucoes2' => 'www.nomefantasia.com' ,
	        			 	'instrucoes3' => '' ,
	        			 	'instrucoes4' => '' ,
	        			 	'quantidade' => '001' ,
	        			 	'valor_unitario' => '100,25' ,
	        			 	'aceite' => '' ,
	        			 	'especie' => 'R$' ,
	        			 	'especie_doc' => 'DS' ,
	        			 	'agencia' => '5689' ,
	        			 	'agencia_dv' => '1' ,
	        			 	'conta' => '06511' ,
	        			 	'conta_dv' => '7' ,
	        			 	'conta_cedente' => '' ,
	        			 	'conta_cedente_dv' => '' ,
	        			 	'convenio' => '123456' ,
	        			 	'contrato' => '123456' ,
	        			 	'formatacao_convenio' => 6 ,
	        			 	'formatacao_nosso_numero' => '2' ,
	        			 	'identificacao' => 'Nome Fantasia' ,
	        			 	'cpf_cnpj' => '65146516' ,
	        			 	'endereco' => 'Rua Endereço 111 Cj 100 Bairro BH MG 41651-65 BR' ,
	        			 	'cidade_uf' => 'BH MG' ,
	        			 	'cedente' => 'Nome registro',
	        			 	'codigo_cliente' =>	'1234567',
	        			 	'ponto_venda' =>	'5689',
	        			 	'carteira_descricao' =>	'COBRANÇA SIMPLES - CSR'
 						)
 					);

        $instantBuyKey = $this->bankSlip->bankSlipTransaction($bankSlip);
	}

	public function testBoletoTransactionFormatLessFiveDigitsOurNumber(){

		$storeAddress = new Address("Rua Endereço", "111", "Cj 100", "Bairro", "BH", "MG", "41651-65", "BR");
		$storeBankInfo = new StoreBankAccount("5689", "1", "06511", "7", "123", "1234567");
		$storeInfo = new Store("Nome Fantasia", $storeAddress, $storeBankInfo, "www.nomefantasia.com", "65146516", "Nome registro", "123456", "123456");

		$bankSlipDifferentOurNumber = new PaymentImpl($storeInfo, $this->requireFiles);

		$clientAddress = new Address("Rua Pernambuco", "1000", "Cj 102", "Funcionários", "Belo Horizonte", "Minas Gerais", "30130-151", "BR");
		$clientInfo = new Customer;
		$clientInfo->setName("Teste Teknisa");
		$clientInfo->setAddress($clientAddress);
		$clientInfo->setDocument("12985019999");
		$clientInfo->setEmail("lucas.lacerda@teknisa.com");
		$clientInfo->setPhone("(31)991664747");
		$bankSlip = new BankSlipTransaction("033", "100.25", "123123123", "654987", "Boleto de testes Teknisa", "5", $clientInfo);

		$this->requireFiles->expects($this->once())
        			 ->method('bankSlipConstruct')
        			 ->with(
        			 	array(
	        			 	'carteira' => '102' ,
	        			 	'bankName' => 'santander' ,
	        			 	'nosso_numero' => '123' ,
	        			 	'numero_documento' => '123123123' ,
	        			 	'data_vencimento' => $this->dateBankSlipPlusFiveDays,
	        			 	'data_documento' => $this->dateBankSlip ,
	        			 	'data_processamento' => $this->dateBankSlip ,
	        			 	'valor_boleto' => '100,25' ,
	        			 	'sacado' => 'Teste Teknisa' ,
	        			 	'endereco1' => 'Rua Pernambuco 1000 Cj 102 Funcionários Belo Horizonte Minas Gerais 30130-151 BR' ,
	        			 	'endereco2' => '' ,
	        			 	'demonstrativo1' => 'Boleto de testes Teknisa' ,
	        			 	'demonstrativo2' => '' ,
	        			 	'demonstrativo3' => '' ,
	        			 	'instrucoes1' => '- Favor nao receber apos o vencimento' ,
	        			 	'instrucoes2' => 'www.nomefantasia.com' ,
	        			 	'instrucoes3' => '' ,
	        			 	'instrucoes4' => '' ,
	        			 	'quantidade' => '001' ,
	        			 	'valor_unitario' => '100,25' ,
	        			 	'aceite' => '' ,
	        			 	'especie' => 'R$' ,
	        			 	'especie_doc' => 'DS' ,
	        			 	'agencia' => '5689' ,
	        			 	'agencia_dv' => '1' ,
	        			 	'conta' => '06511' ,
	        			 	'conta_dv' => '7' ,
	        			 	'conta_cedente' => '' ,
	        			 	'conta_cedente_dv' => '' ,
	        			 	'convenio' => '123456' ,
	        			 	'contrato' => '123456' ,
	        			 	'formatacao_convenio' => 6 ,
	        			 	'formatacao_nosso_numero' => '1' ,
	        			 	'identificacao' => 'Nome Fantasia' ,
	        			 	'cpf_cnpj' => '65146516' ,
	        			 	'endereco' => 'Rua Endereço 111 Cj 100 Bairro BH MG 41651-65 BR' ,
	        			 	'cidade_uf' => 'BH MG' ,
	        			 	'cedente' => 'Nome registro',
	        			 	'codigo_cliente' =>	'1234567',
	        			 	'ponto_venda' =>	'5689',
	        			 	'carteira_descricao' =>	'COBRANÇA SIMPLES - CSR'
 						)
 					);

        $instantBuyKey = $bankSlipDifferentOurNumber->bankSlipTransaction($bankSlip);
	}

	public function testBoletoTransactionUnsupportedBank(){

		$this->setExpectedException(Exception::class);
		$this->expectExceptionMessage("Unsupported bank.");

		$clientAddress = new Address("Rua Pernambuco", "1000", "Cj 102", "Funcionários", "Belo Horizonte", "Minas Gerais", "30130-151", "BR");
		$clientInfo = new Customer;
		$clientInfo->setName("Teste Teknisa");
		$clientInfo->setAddress($clientAddress);
		$clientInfo->setDocument("12985019999");
		$clientInfo->setEmail("lucas.lacerda@teknisa.com");
		$clientInfo->setPhone("(31)991664747");
		$bankSlip = new BankSlipTransaction("unsupportedBank", "100.25", "123123123", "654987", "Boleto de testes Teknisa", "5", $clientInfo);

        $instantBuyKey = $this->bankSlip->bankSlipTransaction($bankSlip);
	}

	public function testFactory(){
		$storeAddress = new Address("Rua Endereço", "111", "Cj 100", "Bairro", "BH", "MG", "41651-65", "BR");
		$storeBankInfo = new StoreBankAccount("5689", "1", "06511", "7", "121456", "1234567");
		$storeInfo = new Store("Nome Fantasia", $storeAddress, $storeBankInfo, "www.nomefantasia.com", "65146516", "Nome registro", "123456", "123456");
		$factory = $this->bankSlip->factory($storeInfo, $this->requireFiles);
		$this->assertInstanceOf(PaymentImpl::class, $factory);
	}

}