ZeedhiChart.directive('zhMorris', function($log, $timeout, eventEngine, maskEngine) {
	return {
		restrict: 'A',
		scope: {
			chartData: '='
		},
		link: function(scope, ele, attrs) {

			var getWidget = function() {
				return scope.$parent.widget;
			};

			var data, options;
			var container = ele.parent();

			var defaultColors = ['#f1c40f', '#f39c12', '#e67e22', '#d35400', '#e74c3c', '#c0392b', '#9b59b6', '#8e44ad', '#2ecc71', '#27ae60', '#1abc9c', '#16a085', '#3498db', '#2980b9', '#34495e', '#2c3e50', '#95a5a6', '#7f8c8d', '#bdc3c7', '#ecf0f1'];

			getWidget().chartColors = getWidget().lineColors || getWidget().chartColors || defaultColors;

			function calcMinWidth() {
				var dataSourceLength = getWidget().dataSource.data ? getWidget().dataSource.data.length : 0;
				var w = (dataSourceLength * ((getWidget().barWidth) / 0.75)) + 88;
				var minw = (w ? (w + 'px') : '100%');
				container.css('min-width', minw);
			}

			ele.mouseout(function() {
				ele.find(".morris-hover").hide();
			});

			if (getWidget()) {
				var events = getWidget().events;
				if (events) {
					eventEngine.bindEvents(
						ele,
						events,
						function() {
							if (getWidget().dataSource) {
								return getWidget().dataSource.data;
							}
						},
						function() {
							return getWidget().currentRow;
						},
						function() {
							return getWidget().selectedRow;
						}
					);
				}
			}

			var _bindOnClick = function() {
				if (attrs.type == "bar") {
					ele.find("rect").each(function(index, elem) {

						var rowIndex = parseInt(index / JSON.parse(attrs.ykeys).length);
						$(elem).attr('row-index', rowIndex);

						if(Zepto.os.phone || Zepto.os.tablet) {
							elem.onmouseover = function() {
								getWidget().setCurrentRow(getWidget().dataSource.data[rowIndex]);
							};
						} else  {
							elem.onclick = function() {
								getWidget().setCurrentRow(getWidget().dataSource.data[rowIndex]);
							};
						}
					});
				} else if (attrs.type == "donut") {
					ele.find("path").each(function(index, elem) {
						elem.onclick = elem.ontouchstart = function() {
							getWidget().setCurrentRow(getWidget().dataSource.data[Math.ceil((index / 2)-1)]);
						};
					});
				} else {
					ele.find("circle").each(function(index, elem) {

						var rowIndex = parseInt(index % getWidget().dataSource.data.length);
						$(elem).attr('row-index', rowIndex);

						elem.onclick = elem.ontouchstart = function() {
							getWidget().setCurrentRow(getWidget().dataSource.data[rowIndex]);
						};
					});
				}
			};

			function _redraw() {
				draw(data);
				_bindOnClick();
			}

			scope.$watch('chartData.data', function(e) {
				data = e;
				calcMinWidth();
				_redraw();
			});

			$(window).resize(function() {
				if (ele.is(":visible")) {
					_redraw();
				}
			});

			getWidget().redraw = function() {
				_redraw();
			};

			$timeout(_redraw, 200);

			var defaultLineClamOpts = {
				lines: 2,
				lineLength: 15,
				breakStr: '\n'
			};

			getWidget().clampOptions = jQuery.extend(defaultLineClamOpts, getWidget().clampOptions);

			function lineClamp(str, lines, lineLength, breakStr) {

				var subStrs = str.split(" ");
				var resultLines = [];

				for(var i=0; i<subStrs.length; i++) {

					var currentStr = subStrs[i];
					var nextStr = subStrs[i+1];

					if(nextStr && (currentStr + " " + nextStr).length > lineLength) {
						resultLines.push(currentStr);
					} else if(nextStr) {
						subStrs[i+1] = currentStr + " " + nextStr;
					} else {
						resultLines.push(currentStr);
					}
				}

				if(resultLines.length > lines) {
					var lastStr = resultLines[lines - 1];
					resultLines[lines - 1] = lastStr.slice(0, lineLength - 3) + "...";
					resultLines.splice(lines, resultLines.length - lines);
				}

				return resultLines.join(breakStr);
			}

			function buildDonutColors(arr){
				var newArray = [], auxColors = Util.clone(getWidget().donutColors || getWidget().chartColors || defaultColors);
				var i, aux = -1;
				for(i in arr){
					if(arr[i].color){
						newArray.push(arr[i].color);
					}
					else if(auxColors.length){
						newArray.push(auxColors.shift());
					}
					else{
						if(!newArray[aux+1]){
							aux = -1;
						}
						newArray.push(newArray[++aux]);
					}
				}
				return newArray.concat(auxColors);
			}

			function draw(e) {
				data = e;
				if (data && data.length) {
					ele.context.innerHTML = "";
					var yLabelFormat = getWidget().yAxisMask ? function(value) {
						var result;
						var params = {};
						var widget = getWidget();
						if (!Util.isUndefined(widget.maxValue)) {
							params.maxValue = widget.maxValue;
						}
						if (!Util.isUndefined(widget.minValue)) {
							params.minValue = widget.minValue;
						}
						if (!Util.isUndefined(widget.valuePrecision)) {
							params.precision = widget.valuePrecision;
						}
						
						var mappedMasks = maskEngine.getMappedMasks();
						var mappedMask = mappedMasks[widget.yAxisMask];
						if (mappedMask && mappedMask.format) {
							result = mappedMask.format.apply(maskEngine, [value, params]);
						} else {
							result = value;
						}
						
						return result;
					} :	null;
					switch (attrs.type) {
						case 'bar':
							var ykeys = JSON.parse(attrs.ykeys);
							options = {
								element: ele[0],
								ykeys: ykeys,
								data: data.map(function(obj){
									for(var i in ykeys){
										if (obj[ykeys[i]]) {
											obj[ykeys[i]] = obj[ykeys[i]].toString().replace(/,/g, '.');
										}
									}
									return obj;
								}),
								xkey: attrs.xkey,
								hideHover: getWidget().hideHover || 'always',
								hoverCallback: function (index, options, content, row) {
								  getWidget().__rowIndex = index;
								  return content;
								},
								gridTextColor: '#000',
								gridTextFamily: 'open_sanslight',
								labels: getWidget().barLabels || JSON.parse(attrs.labels || attrs.ykeys),
								barColors: getWidget().barColors || getWidget().chartColors || defaultColors,
								stacked: attrs.stacked || null,
								resize: false,
								redraw: false,
								xLabelAngle: getWidget().labelAngle || 60,
								parseTime: false,
								xLabelFormat: function(value) { return lineClamp(value.label, getWidget().clampOptions.lines, getWidget().clampOptions.lineLength, getWidget().clampOptions.breakStr); },
								yLabelFormat: yLabelFormat
							};
							return new Morris.Bar(options);
						case 'line':
							options = {
								element: ele[0],
								data: data.map(function(obj){
									for(var i in ykeys){
										if (obj[ykeys[i]]) {
											obj[ykeys[i]] = obj[ykeys[i]].toString().replace(/,/g, '.');
										}
									}
									return obj;
								}),
								xkey: attrs.xkey,
								ykeys: JSON.parse(attrs.ykeys),
								hideHover: getWidget().hideHover || 'always',
								hoverCallback: function (index, options, content, row) {
								  getWidget().__rowIndex = index;
								  return content;
								},
								labels: getWidget().lineLabels || JSON.parse(attrs.labels || attrs.ykeys),
								lineWidth: attrs.lineWidth || 2,
								lineColors: getWidget().lineColors || getWidget().chartColors || defaultColors,
								resize: false,
								redraw: false,
								xLabelAngle: getWidget().labelAngle || 60,
								parseTime: false,
								xLabelFormat: function(value) { return lineClamp(value.label, 2, 12, '\n'); },
								yLabelFormat: yLabelFormat
							};
							return new Morris.Line(options);
						case 'area':
							options = {
								element: ele[0],
								data: data,
								xkey: attrs.xkey,
								ykeys: JSON.parse(attrs.ykeys),
								labels: JSON.parse(attrs.labels || attrs.ykeys),
								hideHover: getWidget().hideHover || 'auto',
								hoverCallback: function (index, options, content, row) {
								  getWidget().__rowIndex = index;
								  return content;
								},
								lineWidth: attrs.lineWidth || 2,
								lineColors: getWidget().areaColors || getWidget().chartColors || defaultColors,
								behaveLikeLine: attrs.behaveLikeLine || false,
								fillOpacity: attrs.fillOpacity || 'auto',
								pointSize: attrs.pointSize || 4,
								resize: false,
								redraw: false,
								xLabelAngle: getWidget().labelAngle || 60,
								parseTime: false,
								xLabelFormat: function(value) { return lineClamp(value.label, 2, 15, '\n'); },
								yLabelFormat: yLabelFormat
							};
							return new Morris.Area(options);
						case 'donut':
							getWidget().donutLegendColors = buildDonutColors(data);
							var chartKey = getWidget().chartKey || 'value';
							var chartLabel = getWidget().chartLabel || 'label';
							var donutData = data.map(function(item){
								var obj = { value: item[chartKey], label: item[chartLabel] };
								if(obj.value) {
									obj.value = obj.value.toString().replace(/,/g, ".");
								}
								return obj;
							});

							options = {
								element: ele[0],
								data: donutData,
								colors: getWidget().donutLegendColors,
								resize: false,
								redraw: false,
								parseTime: false
							};
							var func;
							if (attrs.formatter) {
								/*jslint evil: true */
								func = new Function('y', 'data', attrs.formatter);
								options.formatter = func;
							} else if(attrs.valueFormat || attrs.mask){
								var mask = getWidget().mask;
								func = function(y, data){
									return maskEngine[(getWidget().valueFormat || mask.name) + "Format"](y, mask ? (mask.params || {}) : {});
								};
								options.formatter = func;
							}
							return new Morris.Donut(options);
						default:
							break;
					}
				}
			}

			$timeout(function() {
				ele.find('.morris-hover').click(function() {
					var xKey = getWidget().__rowIndex;
					$(ele.find('[row-index=' + xKey + ']')[0]).click();
				});
			}, 3000);

		}
	};
});

ZeedhiChart.directive("zhChartColumnAndLine", function($timeout) {
	return {
		restrict: 'A',
		link: function($scope, $element, $attrs) {

			var widget = $scope.widget;
			var barChart = $element.find('.zh-bar-chart');
			var lineChart = $element.find(".zh-line-chart");

			var defaultColors = ['#f1c40f', '#f39c12', '#e67e22', '#d35400', '#e74c3c', '#c0392b', '#9b59b6', '#8e44ad', '#2ecc71', '#27ae60', '#1abc9c', '#16a085', '#3498db', '#2980b9', '#34495e', '#2c3e50', '#95a5a6', '#7f8c8d', '#bdc3c7', '#ecf0f1'];

			function createTransparentArray(length) {
				var arr = [];
				for(var i = 0; i < length; i++) {
					arr.push("transparent");
				}
				return arr;
			}

			function isEmptyArray(value) {
				return !Util.isArray(widget.barColors) || !widget.barColors.length;
			}

			if (isEmptyArray(widget.barColors) || widget.barYkeys.length !== widget.barColors.length) {
				widget.barColors = defaultColors.slice(0, widget.barYkeys.length);
			}
			if (isEmptyArray(widget.lineColors) || widget.lineYkeys.length !== widget.lineColors.length) {
				widget.lineColors = defaultColors.slice(widget.barYkeys.length, widget.lineYkeys.length + widget.barYkeys.length);
			}

			// Prepares chart colors to hide
			widget.barColors = widget.barColors.concat( createTransparentArray(widget.lineYkeys.length) );
			widget.lineColors = widget.lineColors.concat( createTransparentArray(widget.barYkeys.length) );

			function drawLinesOnBarChart() {
				$timeout(function() {
					barChart.find("svg").append(lineChart.find("svg path[stroke-width='2']"));
					lineChart.find("svg path[stroke-width='2']").remove();
				}, 200);
			}

			$(window).resize(drawLinesOnBarChart);
			drawLinesOnBarChart();
		}
	};
});
