describe('ContainerDecoratorProvider', function () {

	var screenService = { i18n: function(text) { return text; } };
	var zhPromise = new ZHPromise(Q);
	var requestDataSourceEngine = {
		save: function(){
			return thenable();
		}
	};
	var localStorageEngine = {
		save: function(){
			return thenable();
		},
		getDataSource: function(){
			return thenable([defaultContainerPreference]);
		},
		sync: function(){
			return thenable([defaultContainerPreference]);
		}
	};

	var zhUtil = {
			registerDataSource : function(name){
				repoTest = name;
		}};

	var repositoryFactory = {
		factory: function(){
			return {
				find: function(){
					return zhPromise.when([{"name":"login"}]);
				},
				save: function(){
					return zhPromise.when();
				}
			};
		}
	};

	var synchronizableStorageEngineFactory = new SynchronizableStorageEngineFactory(zhPromise);
	var preferenceManager = new PreferenceManager(synchronizableStorageEngineFactory, {}, localStorageEngine, requestDataSourceEngine, new QueryBuilder(), screenService, zhPromise, zhUtil, repositoryFactory);
	var containerRepository = new ContainerRepository(preferenceManager);
	var containerMerger = new ContainerMerger();
	var containerDecoratorPreference = new ContainerDecoratorPreference(containerRepository, screenService, zhPromise, containerMerger);
	var containerDecoratorProvider = new ContainerDecoratorProvider(containerDecoratorPreference);

	var defaultContainerPreference = {
		"name": "login",
		"showFooter": false,
		"widgets": [
			{
				"id": 1,
				"name": "loginWidget",
				"fields": [{
					"id": 2,
					"label": "Default label"
				}]
			}
		]
	};

	it("The 'provide' method must return all providers to a container", function(){
		var providers = containerDecoratorProvider.provide('login');
		expect(providers.length).toEqual(1);
		expect(providers[0]).toEqual(jasmine.any(ContainerDecoratorPreference));
	});

	it("The 'provide' method must privide the right", function(){
		var providerFake = {};
		containerDecoratorProvider.registerProvider(providerFake);
		var providers = containerDecoratorProvider.provide('login');
		expect(providers.length).toEqual(2);
		containerDecoratorProvider.unregisterProvider(providerFake);
		providers = containerDecoratorProvider.provide('login');
		expect(providers.length).toEqual(1);
		expect(providers[0]).toEqual(jasmine.any(ContainerDecoratorPreference));
	});

});