describe('MetaDataLoader', function() {
	var wizardContainer = {
		"id": 1234,
		"name": "signUpWizard",
		"label": "Sign Up",
		"type": "wizard",
		"currentStageName": "registerInit",
		"events": [{
			"name": "WizardOnCancel",
			"code": "wizardOnCancel1 = true;",
			"id": 2
		}],
		"history": [],
		"stages": {
			"registerInit": {
				"name": "registerInit",
				"window": "registerInit",
				"events": [{
					"id": 1,
					"name": "WizardBeforeNextStage",
					"code": "wizardBeforeNextStage = true;"
				}, {
					"name": "WizardOnCancel",
					"code": "wizardOnCancel2 = true;",
					"id": 2
				}],
				"nextStages": [{
					"name": "registerAddress",
					"route": "registerAddress"
				}]
			},
			"registerAddress": {
				"name": "registerAddress",
				"window": "registerAddress",
				"events": [{
					"name": "WizardOnDecision",
					"code": "decisionWizard1();"
				}],
				"nextStages": [{
					"name": "registerNews",
					"route": "registerNews"
				}, {
					"name": "registerEnd",
					"route": "registerEnd"
				}]
			},
			"registerNews": {
				"name": "registerNews",
				"window": "registerNews",
				"events": [{
					"id": 1,
					"name": "WizardOnFinish",
					"code": "wizardOnFinishNews = true;"
				}],
				"nextStages": []
			},
			"registerEnd": {
				"name": "registerEnd",
				"window": "registerEnd",
				"events": [{
					"id": 1,
					"name": "WizardOnFinish",
					"code": "wizardOnFinishEnd = true;"
				}],
				"nextStages": []
			}
		}
	};

	var defaultContainer = {
		"name": "login",
		"showFooter": true,
		"showMenu": true,
		"widgets": [
			{
				"id": 1,
				"name": "loginWidget",
				"template": "widget/list.html",
				"actions": [
					{
						"id": 2,
						"name": "TEST",
						"showAsAction": "always",
						"events": [
							{
								"id": 3,
								"name": "ActionEvent",
								"code": ";"
							}
						]
					}
				],
				"events": [
					{
						"id": 4,
						"name": "WidgetOnEnter",
						"code": ";"
					}
				],
				"fields": [
					{
						"id": 5,
						"name": "TEST",
						"label": "Default label"
					}
				],
				"styles": [
					{
						"id": 6,
						"condition": "row.id % 2 == 0",
						"cssClass": "fuck"
					}
				],
				"fieldGroups": [
					{
						"id": 7,
						"name": "ORG",
						"label": "Dados da organização",
						"collapsible": true,
						"opened": true
					}
				],
				"widgets": [
					{
						"id": 100,
						"name": "secondWidget",
						"label": "Second widget",
						"events": [
							{
								"id": 102,
								"name": "WidgetOnEnter",
								"code": ";"
							}
						]
					}
				]
			}
		]
	};

	var	defaultContainerPreference = {
		"name": "login",
		"showFooter": false,
		"widgets": [
			{
				"id": 1,
				"name": "loginWidget",
				"template": "widget/form.html",
				"actions": [
					{
						"id": 2,
						"name": "TEST",
						"showAsAction": "never",
						"events": [
							{
								"id": 3,
								"name": "ActionEvent",
								"code": "args.owner.widget.newRow();"
							}
						]
					}
				],
				"events": [
					{
						"id": 4,
						"name": "WidgetOnEnter",
						"code": "args.owner.newRow();"
					}
				],
				"fields": [
					{
						"id": 5,
						"name": "TEST",
						"label": "Preference label"
					},{
						"id": 20,
						"name": "NEW_FIELD",
						"label": "New field"
					}
				],
				"styles" : [
					{
						"id": 6,
						"condition" : "row.ID % 2 === 0",
						"cssClass" : "zh-inactive"
					}
				],
				"fieldGroups": [
					{
						"id": 7,
						"name": "ORG changed",
						"label": "Dados da organização",
						"collapsible": true,
						"opened": false
					}
				],
				"widgets": [
					{
						"id": 1000,
						"name": "thirdWidget",
						"label": "3"
					},
					{
						"id": 100,
						"name": "secondWidget",
						"label": "2",
						"fields": [
							{
								"id": 101,
								"name": "second TEST",
								"label": "Second widget field label"
							}
						],
						"events": [
							{
								"id": 102,
								"name": "WidgetOnEnter",
								"code": "args.owner.newRow();"
							}
						]
					}
				]
			}
		]
	};

	var zhPromise = new ZHPromise(Q);
	var screenService = {
		i18n: function(text){ return text; },
		isPopupOpen: function() { return false; }
	};
	var templateManager = {
		"containers": {},
		"projectPromise": zhPromise.when(),
		"project": {
				"containersTimeToLive": 1
		},
		getMappedPackages: function(){
			return {
				default: ["./"]
			};
		},
		historyItems: []
	};
	var thenable = function(data) {
		return {
			then : function(fn){
				var dataReturned;
				if(fn){
					dataReturned = fn(data);
				}
				return thenable(dataReturned || data);
			}
		};
	};

	var requestDataSourceEngine = {
		save: function(){
			return thenable();
		}
	};
	var localStorageEngine = {
		save: function(){
			return thenable();
		},
		getDataSource: function(){
			return thenable([defaultContainerPreference]);
		},
		sync: function(){
			return thenable([defaultContainerPreference]);
		}
	};
	var restEngine = new RestEngine(console, function(p) {
		lastParams = p;
		var containerName = lastParams.url.substr(0 , lastParams.url.length - 5).split('/');
		containerName = containerName[containerName.length - 1];
		return {
			success: function(fn) {
				if(fn) {
					if(containerName == "login"){
						fn(defaultContainer);
						return thenable(defaultContainer);
					} else if(containerName == "signUpWizard"){
						fn(wizardContainer);
						return thenable(wizardContainer);
					}
				}
			}
		};
	}, templateManager, {}, new ZhFeatures(console), zhPromise, {
		onRequestStart: function() {},
		onRequestMetaDataStart: function() {}
	}, new RequestInterceptingEngine(zhPromise));

	var zhUtil = {
		registerDataSource : function(name){
			repoTest = name;
	}};

	var repositoryFactory = {
		factory: function(){
			return {
				find: function(){
					return zhPromise.when([defaultContainerPreference]);
				},
				save: function(){
					return zhPromise.when();
				}
			};
		}
	};

	var fallbackStorageStrategy = {};
	var SQLStorageStrategy = {};
	var aclEngine = new AclEngine();
	var eventEngine = new EventEngine(new EventFactory(new MethodBuilder({}, {} ), {}));
	var metaDataFactory = new MetaDataFactory(localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine, eventEngine, screenService, aclEngine, { project: {}},{},zhPromise, {}, {}, {}, {}, {}, {}, {});
	var synchronizableStorageEngineFactory = new SynchronizableStorageEngineFactory(zhPromise);
	var preferenceManager = new PreferenceManager(synchronizableStorageEngineFactory, fallbackStorageStrategy, localStorageEngine, requestDataSourceEngine, new QueryBuilder(), screenService, zhPromise, zhUtil, repositoryFactory);
	var containerRepository = new ContainerRepository(preferenceManager);
	var containerMerger = new ContainerMerger();
	var containerDecoratorPreference = new ContainerDecoratorPreference(containerRepository, screenService, zhPromise, containerMerger);
	var containerDecoratorProvider = new ContainerDecoratorProvider(containerDecoratorPreference);
	var jsonContainerLoader = new JSONContainerLoader(restEngine, zhPromise, screenService);
	var containerLoader = new ContainerLoader(jsonContainerLoader, containerDecoratorProvider, zhPromise, screenService);
	var metaDataLoader = new MetaDataLoader(containerLoader, metaDataFactory, screenService, zhPromise, templateManager, restEngine);

	it("Method 'loadContainer': Should provide loadingContainerJson only during container factory", function(done){
		spyOn(metaDataFactory, 'containerFactory').and.callFake(function() {
			expect(templateManager.loadingContainerJson).toBe(defaultContainer);
		});

		metaDataLoader.loadContainer('login').then(function(container){
			expect(metaDataFactory.containerFactory).toHaveBeenCalled();
			expect(templateManager.loadingContainerJson).toBe(null);
		}).then(function() {
			done();
		});
	});

	it("Method 'loadContainer': Should return a rejected promise if doen't pass parameters", function(done){
		var loadError1 = false,
		    loadError2 = false,
		    promises = [];

		promises.push(metaDataLoader.loadContainer().then(null, function(){ loadError1 = true; }));
		promises.push(metaDataLoader.loadContainer(null).then(null, function(){ loadError2 = true; }));

		zhPromise.all(promises).then(function(){
			expect(loadError1).toBe(true);
			expect(loadError2).toBe(true);
			done();
		});

	});

	it("Method 'loadContainer': Should return the container", function(done){
		metaDataLoader.loadContainer('login').then(function(container){
			expect(container).toEqual(jasmine.any(Container));
			expect(container.showFooter).toEqual(false);
			expect(container.showMenu).toEqual(true);
			expect(container.widgets[0].template).toEqual('widget/form.html');
			expect(container.widgets[0].actions[0].showAsAction).toEqual('never');
			expect(container.widgets[0].actions[0].events[0].code).toEqual('args.owner.widget.newRow();');
			expect(container.widgets[0].events[0].code).toEqual('args.owner.newRow();');
			expect(container.widgets[0].fields.length).toEqual(2);
			expect(container.widgets[0].fields[0].label).toEqual('Preference label');
			expect(container.widgets[0].fields[1].name).toEqual('NEW_FIELD');
			expect(container.widgets[0].fields[1].label).toEqual('New field');
			expect(container.widgets[0].styles[0].cssClass).toEqual('zh-inactive');
			expect(container.widgets[0].fieldGroups[0].name).toEqual('ORG changed');
			expect(container.widgets[0].fieldGroups[0].opened).toEqual(false);
			expect(container.widgets[0].widgets.length).toEqual(2);
			expect(container.widgets[0].widgets[0].id).toEqual(100);
			expect(container.widgets[0].widgets[0].name).toEqual('secondWidget');
			expect(container.widgets[0].widgets[0].label).toEqual('2');
			expect(container.widgets[0].widgets[0].fields.length).toEqual(1);
			expect(container.widgets[0].widgets[0].events.length).toEqual(1);
			expect(container.widgets[0].widgets[0].events[0].id).toEqual(102);
			expect(container.widgets[0].widgets[0].events[0].code).toEqual('args.owner.newRow();');
			expect(container.widgets[0].widgets[1].id).toEqual(1000);
			expect(container.widgets[0].widgets[1].name).toEqual('thirdWidget');
			expect(container.widgets[0].widgets[1].label).toEqual('3');

			done();
		});

	});

	it("Method 'loadContainer': If the container was loaded, the MetaDataLoader should return the container", function(done){
		metaDataLoader.loadContainer('login').then(function(container){
			container.name = "Container changed";
			metaDataLoader.loadContainer('login').then(function(container2){
				expect(container2.name).toEqual("Container changed");
				done();
			});
		});
	});

	it("Container shouldn't exists one second after created ", function(done){
		metaDataLoader.loadContainer('login').then(function(){
			setTimeout(function(){
				expect(templateManager.containers.login).toEqual(undefined);
				done();
			}, 1500);
		});
	});

	it("Wizard container should be factored by WizardFactory", function(done){
		metaDataLoader.loadContainer('signUpWizard').then(function(){
			expect(templateManager.containers.signUpWizard.type).toEqual("wizard");
			done();
		});
	});

	it('Method loadMessagesFile should load messages.json file', function() {
		spyOn(restEngine, 'requestMetaData').and.callFake(function() {
			return zhPromise.when({
				data: {
					'CODE1': 'Message 1',
					'CODE2': 'Message 2'
				}
			});
		});

		metaDataLoader.loadMessagesFile('custom_package');
		expect(restEngine.requestMetaData).toHaveBeenCalledWith('custom_package#json/messages.json');
	});

});