describe("CarouselController", function () {
  var carouselController;
	var $scope;
	var $rootScope;
	var container;
	var widget;
	
	var baseContainer = ZhHelper.baseContainerWithTwoWidgets;
	
	beforeEach(function () {
		module(function ($provide) {
			//Providing a new $timeout because $timeout.flush is not flushing pending $timeout calls
			$provide.constant("$timeout", setTimeout);
			$provide.constant("$q", Q);
		});
		module("ZeedhiApp");
		module("ZeedhiServices");
		
		inject(function (_$rootScope_, $controller, metaDataFactory) {
			container = baseContainer();
			container.widgets[0].template = "widget/carosuel.html";
			container.widgets[0].fields = [];
			container = metaDataFactory.containerFactory(container);
			widget = container.widgets[0];
			$rootScope = _$rootScope_;
			
			$scope = _$rootScope_.$new();
			$scope.$parent = _$rootScope_;
			$scope.widget = widget;
			
			carouselController = $controller("CarouselController", {
				$scope: $scope,
				$rootScope: $rootScope
			});
		});
	});
	
	it("carouselWidget.getPreviousIndex: should return the currentIndex -1 if the carouselWidget has less than 2 widgets", function () {
	  widget.widgets = widget.widgets.slice(0, 1);
	  expect(widget.getPreviousIndex()).toEqual(-1);
	});
	
	it("carouselWidget.getPreviousIndex: return the carouselWidget.widgets.length -1 if the currentIndex equals 0 and the carouselWidget has 2 widgets or more", function () {
	  expect(widget.getPreviousIndex()).toEqual(widget.widgets.length -1);
	});
	
	it("carouselWidget.getNextIndex: should return 0 if the currentIndex equals to the last widget", function () {
	  widget.widgets = widget.widgets.slice(0, 1);
	  expect(widget.getNextIndex()).toEqual(0);
	});
	
	it("carouselWidget.getNextIndex: should return the currentIndex + 1 if the currentIndex does not equals to the last widget", function () {
	  expect(widget.getPreviousIndex()).toEqual(1);
	});
	
	it("carouselWidget.nextWidget: should set the carouselWidget label to the selected widget and activate the widget", function () {
	  var nextWidget =  widget.widgets[1];
	  nextWidget.label = "42";
	  $scope.nextWidget = function () {};
	  
	  spyOn($scope, "nextWidget").and.callThrough();
	  widget.nextWidget();
	  expect($scope.nextWidget).toHaveBeenCalled();
	  expect(widget.label).toEqual("42");
	  expect($rootScope._currentPopup).toEqual(nextWidget);
	  expect(nextWidget.isActive()).toBeTruthy();
	});
	
	it("carouselWidget.previousWidget: should work as with nextWidget, but going backwards", function () {
	  var nextWidget =  widget.widgets[1];
	  var previousWidget = widget.widgets[0];
	  previousWidget.label = "What do you get when you multiply six by nine?";
	  nextWidget.label = "42";
	  $scope.nextWidget = function () {};
	  
	  spyOn($scope, "nextWidget").and.callThrough();
	  widget.nextWidget();
	  expect($scope.nextWidget).toHaveBeenCalled();
	  expect(widget.label).toEqual("42");
	  expect($rootScope._currentPopup).toEqual(nextWidget);
	  expect(nextWidget.isActive()).toBeTruthy();
	  
	  $scope.previousWidget = function () {};
	  
	  spyOn($scope, "previousWidget").and.callThrough();
	  widget.previousWidget();
	  expect($scope.previousWidget).toHaveBeenCalled();
	  expect(widget.label).toEqual(previousWidget.label);
	  expect($rootScope._currentPopup).toEqual(previousWidget);
	  expect(previousWidget.isActive()).toBeTruthy();
	  expect(nextWidget.isActive()).toBeFalsy();
	});
});