describe('GridController', function(){

	var gridController;
	var scopeMock;
	window.templateUrls = [{
		package: "default",
		baseUrl: ""
	}];
	var $injector = {
		get: function(dependencyName) {
			var dependency;
			switch (dependencyName) {
				case "ApplicationContext":
					dependency = {
						reportsRepository: reportsRepository
					};
					break;
				default:
					dependency = {};
					break;
			}
			return {};
		}
	};
	beforeEach(function() {
		module("ZeedhiApp");
		inject(function($rootScope, $window, $controller, _DecodeValueFieldService_) {
			var screenService = ScreenService;
			var zhFeatures = new ZhFeatures({
				templateManager: {
					project: {}
				}
			});
			var templateManagerMock = {
				project: {},
				getFrameworkTemplateURL: function(template) {
					return 'templates/' + template;
				}
			};

			scopeMock = $rootScope.$new();
			scopeMock.$parent = $rootScope;
			scopeMock.widget = {
				showCheckbox: true,
				floatingControl: false,
				searchList: {},
				getActionsByVisibility: function() {
					return [];
				},
				getDataSourceFilter: function() {
					return [];
				},
				actions: [],
				fields: [
					{
						'name': 'field1',
						'inlineEditable': 'number.html',
						'showOnList': true,
						'sequenceInList': 1
					},
					{
						'name': 'field2',
						'showOnList': true,
						'sequenceInList': 2
					},
					{
						'name': 'field3',
						'showOnList': false,
						'sequenceInList': 3
					},
					{
						'name': 'field4',
						'showOnList': true,
						'sequenceInList': 4,
						'fixed': true
					}
				],
				container: {
					actions: []
				},
				dataSourceFilter: []
			};

			gridController = $controller("GridController", {
				$scope: scopeMock,
				ScreenService: {
					i18n: function(word) {return word;}
				},
				templateManager: templateManagerMock,
				ZHPromise: {},
				eventEngine: {},
				Query: {},
				CrosstabService: {},
				maskEngine: {},
				ZhUtil: {
					getOrderByFilter: function() {
						return [];
					}
				},
				ApplicationContext: {},
				ZhFeatures: zhFeatures,
				eventAggregator: {
					publish: function(name, parameters) {}
				},
				DecodeValueFieldService: DecodeValueFieldService
			});
		});
	});

	it('isChildrenWithCheckbox: should return false if row does not have a group header.', function(){
		var rowMock = { __groupHeaders: [] };
		expect(gridController.isChildrenWithCheckbox(rowMock)).toBeFalsy();

		rowMock = {};
		expect(gridController.isChildrenWithCheckbox(rowMock)).toBeFalsy();
	});

	it('isChildrenWithCheckbox: should return false if widget does not have "showCheckbox" property.', function(){
		scopeMock.widget.showCheckbox = false;
		var rowMock = { __groupHeaders: [{}, {}] };
		expect(gridController.isChildrenWithCheckbox(rowMock)).toBeFalsy();
	});

	it('calcLeftPaddingForChildrenRow: should return 25px multiplied by number ' +
	   'of headers.', function(){
		scopeMock.widget.showCheckbox = false;
		var rowMock = { __groupHeaders: [{}, {}] };
		expect(gridController.calcLeftPaddingForChildrenRow(rowMock, 0)).toEqual({'padding-left': 54});
	});

	it('calcLeftPaddingForChildrenRow: should return 25px multiplied by number ' +
	   'of headers and subtracted 30px.', function(){
		var rowMock = { __groupHeaders: [{}, {}] };
		expect(gridController.calcLeftPaddingForChildrenRow(rowMock, 0)).toEqual({'padding-left': 32});
	});

	it('getStripeFooterTemplate: returns the widget template.', function(){
		var footerStripeWidgetMock = { template: 'widget/template.html' };
		expect(gridController.getStripeFooterTemplate(footerStripeWidgetMock)).toBe('templates/widget/template.html');
	});

	it('getStripeFooterTemplate: returns an empty string when there is not widget.', function(){
		var footerStripeWidgetMock;
		expect(gridController.getStripeFooterTemplate(footerStripeWidgetMock)).toBe('');
	});

	it('getStripeFooterTemplate: returns an empty string when there is not widget.template.', function(){
		var footerStripeWidgetMock = {};
		expect(gridController.getStripeFooterTemplate(footerStripeWidgetMock)).toBe('');
	});

	it('Should create cancel and save actions for inline edit if widget has inline editable fields', function() {
		expect(gridController.widget.actions.length).toBe(2);

		expect(gridController.widget.actions[0].name).toEqual('cancel');
		expect(gridController.widget.actions[0].isVisible).toEqual(true);
		expect(gridController.widget.actions[0].label).toEqual('Cancel');
		expect(gridController.widget.actions[0].showAsAction).toEqual('back_inline_edit');
		expect(gridController.widget.actions[0].events).toEqual([{
			'name': 'ActionEvent',
			'code': 'args.owner.widget.cancelActionClick();'
		}]);

		expect(gridController.widget.actions[1].name).toEqual('save');
		expect(gridController.widget.actions[1].isVisible).toEqual(true);
		expect(gridController.widget.actions[1].label).toEqual('Save changes');
		expect(gridController.widget.actions[1].showAsAction).toEqual('never_inline_edit');
		expect(gridController.widget.actions[1].events).toEqual([{
			'name': 'ActionEvent',
			'code': 'args.owner.widget.saveActionClick();'
		}]);
	});

	it('Should not have a floating control', function() {
		expect(scopeMock.widget.floatingControl).toBe(false);
	});

	it('widget._filteredFields should contain the fields ordered and filtered', function() {
		expect(scopeMock.widget._filteredFields.length).toBe(3);
		expect(scopeMock.widget._filteredFields[0].name).toEqual('field4');
		expect(scopeMock.widget._filteredFields[1].name).toEqual('field1');
		expect(scopeMock.widget._filteredFields[2].name).toEqual('field2');

		scopeMock.widget.fields[2].showOnList = true;
		scopeMock.widget.fields[3].fixed = false;
		scopeMock.widget.updateFilteredFields();

		expect(scopeMock.widget._filteredFields.length).toBe(4);
		expect(scopeMock.widget._filteredFields[0].name).toEqual('field1');
		expect(scopeMock.widget._filteredFields[1].name).toEqual('field2');
		expect(scopeMock.widget._filteredFields[2].name).toEqual('field3');
		expect(scopeMock.widget._filteredFields[3].name).toEqual('field4');
	});
});