describe("StripeController", function () {
	var stripeController;
	var metaDataFactory;
	var zhPromise;
	var eventEngine;
	var $scope;
	var $rootScope;
	var screenService;
	var container;
	var widget;
	
	var baseContainer = ZhHelper.baseContainerWithTwoWidgets;
	var baseFields = function () {
		return [
			{
				name : "name",
				isVisible: true,
				aclIsVisible: true,
				showOnList: true,
				skipSearch: false,
				class: 12,
				breakline: true
			},
			{
				name: "birth",
				isVisible: true,
				aclIsVisible: true,
				showOnList: true,
				skipSearch: false,
				class: 12,
				breakline: true
			},
			{
				name: "genre",
				isVisible: true,
				aclIsVisible: true,
				showOnList: true,
				skipSearch: false,
				class: 12,
				breakline: true
			}
		];
	};
	
	beforeEach(function () {
		module(function ($provide) {
			//Providing a new $timeout because $timeout.flush is not flushing pending $timeout calls
			$provide.constant("$timeout", setTimeout);
			$provide.constant("$q", Q);
		});
		module("ZeedhiApp");
		module("ZeedhiServices");
		
		inject(function (_$rootScope_, $controller, _ScreenService_, _metaDataFactory_, _ZHPromise_, _eventEngine_) {
			metaDataFactory = _metaDataFactory_;
			screenService = _ScreenService_;
			zhPromise = _ZHPromise_;
			eventEngine = _eventEngine_;
			container = baseContainer();
			container.widgets[0].template = "widget/stripe.html";
			container.widgets[0].fields = [];
			container = metaDataFactory.containerFactory(container);
			widget = container.widgets[0];
			$rootScope = _$rootScope_;
			
			$scope = _$rootScope_.$new();
			$scope.$parent = _$rootScope_;
			$scope.widget = widget;
			
			stripeController = $controller("StripeController", {
				$scope: $scope,
				$rootScope: $rootScope
			});
		});
	});
	
	it("stripeController.widget: should return the $scope.widget", function () {
		expect(stripeController.widget).toEqual(widget);
	});
	
	it("buildStripeFields: should set to the stripeRows to the visible rows if the widget is not collapsible", function () {
	  widget.collapsible = false;
	  widget.fields = baseFields();
	  widget.fields.forEach(function (field) {
	    field.showWhenCollapsed = true;
	  });
	  $scope.$apply();
	  expect($scope.stripeRows.length).toBe(4);
	  expect($scope.stripeRows[0].fields.length).toBe(0);
	  expect($scope.stripeRows[1].fields[0]).toBeDefined();
	  expect($scope.stripeRows[1].fields[0]).toEqual(widget.fields[0]);
	  expect($scope.stripeRows[2].fields[0]).toBeDefined();
	  expect($scope.stripeRows[2].fields[0]).toEqual(widget.fields[1]);
	  expect($scope.stripeRows[3].fields[0]).toBeDefined();
	  expect($scope.stripeRows[3].fields[0]).toEqual(widget.fields[2]);
	});
	
	it("buildStripeFields: should initially set stripeRows to the always visible rows", function () {
	  widget.collapsible = true;
	  widget.fields = baseFields();
	  widget.fields.forEach(function (field) {
	    field.showWhenCollapsed = true;
	  });
	  $scope.$apply();
	  
	  expect($scope.collapsed).toBeTruthy();
	  expect($scope.stripeRows.length).toBe(2);
	  expect($scope.stripeRows[0].fields.length).toBe(0);
	  expect($scope.stripeRows[1].fields[0]).toBeDefined();
	  expect($scope.stripeRows[1].fields[0]).toEqual(widget.fields[0]);
	});
	
	it("$scope.collapseRows: should change the stripeRows to the always visibleRows if the stripe is collapsed, otherwise should change to the visible rows", function () {
	  widget.collapsible = true;
	  widget.fields = baseFields();
	  widget.fields.forEach(function (field) {
	    field.showWhenCollapsed = true;
	  });
	  $scope.$apply();
	  
	  expect($scope.collapsed).toBeTruthy();
	  expect($scope.stripeRows.length).toBe(2);
	  expect($scope.stripeRows[0].fields.length).toBe(0);
	  expect($scope.stripeRows[1].fields[0]).toBeDefined();
	  expect($scope.stripeRows[1].fields[0]).toEqual(widget.fields[0]);
	  
	  $scope.collapseRows();
	  expect($scope.collapsed).toBeFalsy();
	  expect($scope.stripeRows.length).toBe(4);
	  expect($scope.stripeRows[0].fields.length).toBe(0);
	  expect($scope.stripeRows[1].fields[0]).toBeDefined();
	  expect($scope.stripeRows[1].fields[0]).toEqual(widget.fields[0]);
	  expect($scope.stripeRows[2].fields[0]).toBeDefined();
	  expect($scope.stripeRows[2].fields[0]).toEqual(widget.fields[1]);
	  expect($scope.stripeRows[3].fields[0]).toBeDefined();
	  expect($scope.stripeRows[3].fields[0]).toEqual(widget.fields[2]);
	  
	  $scope.collapseRows();
	  expect($scope.stripeRows.length).toBe(2);
	  expect($scope.stripeRows[0].fields.length).toBe(0);
	  expect($scope.stripeRows[1].fields[0]).toBeDefined();
	  expect($scope.stripeRows[1].fields[0]).toEqual(widget.fields[0]);
	});
});