function NutriReportController(templateManager, metaDataFactory, ScreenService, NutriReportService, LoginRepository) {
	var self = this;

    var high;
    var moderate;
    var low;
    var unclassified;
    var student;
    var store;

	/*Nutritional Report - Store Nutritional*/
	this.startFilterForNutritionalData = function(fistChartWidget, secondChartWidget){
        if (window.lang == "en_us"){
            high = "High";
            moderate = "Moderate";
            low = "Low";
            unclassified = "Unclassified";
            student = "Student";
            store = "Snackbar";
        }
        else {
            high = "Alto";
            moderate = "Moderado";
            low = "Baixo";
            unclassified = "Não Classificado";
            student = "Aluno";
            store = "Lanchonete";
        }
		LoginRepository.findAll().then(filterNutritionalData.bind(null, fistChartWidget, secondChartWidget));
	};

	var filterNutritionalData = function(firstChartWidget, secondChartWidget, loginData){
		var CDFILIAL = loginData[0].CDFILIAL;

		var dashboardWidget = firstChartWidget.parent;

		self.setDefaultPeriod(dashboardWidget);
		var DTINI = dashboardWidget.startDate;
		var DTFIM = dashboardWidget.endDate;

		self.getNutritionalData(firstChartWidget, secondChartWidget, DTINI, DTFIM, CDFILIAL);
	};

	self.getNutritionalData = function(firstChartWidget, secondChartWidget, DTINI, DTFIM, CDFILIAL){
		self.resetDynamicWidgets(firstChartWidget.parent, 3);
		var dashboardWidget = templateManager.container.getWidget('dashboardNutriVision');

		NutriReportService.getNutritionalData(CDFILIAL, DTINI, DTFIM, 'nutritional').then(function(nutritionalData){
			nutritionalData = nutritionalData[0];
			if(!nutritionalData.error){
				// @todo work with N families00000000000000
				// --- Works with one family
				nutritionalData = nutritionalData.data[0];
				if(!nutritionalData.products.error){
					let products = nutritionalData.products.products;
					let chartData = self.calculateNutritionalData(products);

					firstChartWidget.dataSource.data = chartData.mediumAndHighProducts;
					secondChartWidget.dataSource.data = chartData.notClassified;

					let chartWidgets = self.buildCharts(products, firstChartWidget);
					chartWidgets.forEach(function(widget){
						let container = widget.container;
						let newWidget = metaDataFactory.widgetFactory(widget);
						dashboardWidget.widgets.push(newWidget);
					});

					firstChartWidget.isVisible = true;
					secondChartWidget.isVisible = true;
				} else {
					firstChartWidget.isVisible = false;
					secondChartWidget.isVisible = false;
				}

				templateManager.updateTemplate().then(function(){
					self.calculateGridTop();
				});
			}else{
			    firstChartWidget.isVisible  = false;
				secondChartWidget.isVisible = false;
				ScreenService.showMessage(nutritionalData.message);
			}

		});
	};

	this.calculateNutritionalData = function(nutritionalData){
		var chartData = {};

		chartData.mediumAndHighProducts = [];
		chartData.notClassified = [];

		let moderateProducts = nutritionalData.filter(function(item){
			return item.VRNUTRIPROD == 2;
		});

		let highProducts = nutritionalData.filter(function(item){
			return item.VRNUTRIPROD == 1;
		});

		let notClassified = nutritionalData.filter(function(item){
			return item.VRNUTRIPROD == 0;
		});

		let moderatePercentage = (100 * moderateProducts.length) / nutritionalData.length;
		let highPercentage = (100 * highProducts.length) / nutritionalData.length;

		chartData.mediumAndHighProducts.push({name: moderate, quantity: moderatePercentage, color: "#f5b60a"});
		chartData.mediumAndHighProducts.push({name: high, quantity: highPercentage, color: "#41d024"});
		// chartData.mediumAndHighProducts.push({name: "Todos os produtos", quantity: nutritionalData.length});

		let notClassifiedPercentage = (100 * notClassified.length) / nutritionalData.length;

		chartData.notClassified.push({name: unclassified, quantity: notClassifiedPercentage, color: "#808080"});

		return chartData;
	};

	this.buildCharts = function(nutritionalData, widget){
		var groups = [];
		var widgets = [];
		nutritionalData.filter(function(product){
			if(!groups[product.CDGRUPO]){
				groups[product.CDGRUPO] = {NMGRUPO: product.NMGRUPO, data: []};
				groups[product.CDGRUPO].data[0] = {name: unclassified, quantity: 0, color: "#808080"}; //Not Classified
				groups[product.CDGRUPO].data[1] = {name: high, quantity: 0, color: "#41d024"}; //High nutritional value
                groups[product.CDGRUPO].data[2] = {name: moderate, quantity: 0, color: "#f5b60a"}; //Medium nutritinal value
                groups[product.CDGRUPO].data[3] = {name: low, quantity: 0, color: "#ef1111"}; //Low nutritional value
		    }

			switch(product.VRNUTRIPROD){
	            case "0":
					groups[product.CDGRUPO].data[0].quantity++;
					break;
	            case "1":
					groups[product.CDGRUPO].data[1].quantity++;
					break;
				case "2":
					groups[product.CDGRUPO].data[2].quantity++;
					break;
				case "3":
					groups[product.CDGRUPO].data[3].quantity++;
	         		break;
	         	default:
	         		groups[product.CDGRUPO].data[0].quantity++;
	         		break;
	         }
		});

		for(var index in groups){
			let notClassifiedQuantity = groups[index].data[0].quantity;
            let highNutriQuantity = groups[index].data[1].quantity;
            let mediumNutriQuantity = groups[index].data[2].quantity;
			let lowNutriQuantity = groups[index].data[3].quantity;

			let total = notClassifiedQuantity + lowNutriQuantity + mediumNutriQuantity + highNutriQuantity;
			let notClassifiedPercentage = (100 * notClassifiedQuantity) / total;
			let highNutriPercentage = (100 * highNutriQuantity) / total;
			let mediumNutriPercentage = (100 * mediumNutriQuantity) / total;
			let lowNutriPercentage = (100 * lowNutriQuantity) / total;

			groups[index].data[0].quantity = notClassifiedPercentage;
			groups[index].data[1].quantity = highNutriPercentage;
			groups[index].data[2].quantity = mediumNutriPercentage;
			groups[index].data[3].quantity = lowNutriPercentage;

			let builtWidget = self.buildGroupChart(groups[index], widget, index);
		 	widgets.push(builtWidget);
		}

		return widgets;
	};

	this.buildGroupChart = function(group, widget, id){
		//var id = Date.now().toString();
		var newWidget = {
			"id": id,
            "width": 1.5,
            "height": 1.5,
            "color": "#e4e4e4",
            "isVisible": true,
            "valueFormat": "percent",
            "container": widget.container,
	       	"parentWidget": widget.parent,
            "name": group.NMGRUPO,
            "label": group.NMGRUPO,
            "template": "widget/dashboard/chart.html",
            "chartType": "donut",
            "fields": [],
            "events": [],
            "dataSource": {
                "data": group.data
            },
            "actions": [],
            "chartLabels": [],
            "chartLabel": "name",
            "chartKey": "quantity",
            "colorField": "color",
            "widgets": []
        };

        return newWidget;
	};

	/*magicNumber = Number of fields that are not created dynamically*/
	this.resetDynamicWidgets = function(widget, magicNumber){
		var lengthConstant = widget.widgets.length;
		/* Wow this amazing logic. */
		for (var i = magicNumber; i < lengthConstant; ++i){
			widget.widgets.splice(magicNumber, 1);
		}
	};

	this.setDefaultPeriod = function(widget){
		var stripeWidget = widget.container.getWidget('stripe');
		var date = new Date();

		var finalDateInMoment = moment(date);
		var initialDateInMoment = moment(date).subtract(1,'month');

		var finalDateToString = finalDateInMoment.format('DD/MM/YYYY');
		var initialDateToString = initialDateInMoment.format('DD/MM/YYYY');


		if (widget.endDate.length == 0) widget.endDate = finalDateToString ; /* End date. */
	    if (widget.startDate.length == 0) widget.startDate = initialDateToString; /* Start date. */

		stripeWidget.getField('DTINI').label = widget.startDate;
		stripeWidget.getField('DTFIM').label = widget.endDate;
	};

	this.openPeriodSelect = function(widget){
		var dashboardWidget = widget.parent;
		var dataInicial = widget.fields[0];
		var dataFinal = widget.fields[1];
		var currentDate = new Date();
		var maxDate = currentDate.getFullYear() + '-' + (currentDate.getMonth()+1) + '-' + currentDate.getDate();

		dataInicial['datetime-maxDate'] = maxDate;
		dataFinal['datetime-maxDate'] = maxDate;
		dataInicial['datetime-defaultDate'] = currentDate.getFullYear()-1 + '-01-01 23:59';
		dataFinal['datetime-defaultDate'] = currentDate.getFullYear()-1 + '-01-01 23:59';
		dataInicial.setValue(dashboardWidget.startDate);
		dataFinal.setValue(dashboardWidget.endDate);

		ScreenService.openPopup(widget);
	};

	this.setPeriodNutri = function(firstChartWidget, secondChartWidget, DTINI, DTFIM){
		var stripeWidget = templateManager.container.getWidget('stripe');
		var dashboardWidget = templateManager.container.getWidget('dashboardNutriVision');
		var dtIni = moment(DTINI, 'DD/MM/YYYY');
		var dtFim = moment(DTFIM, 'DD/MM/YYYY');

		if (dtFim.diff(dtIni) >= 0){
			dashboardWidget.startDate = DTINI;
			dashboardWidget.endDate = DTFIM;
			ScreenService.closePopup();
			stripeWidget.activate();
			self.limitProductsByPeriod(firstChartWidget, secondChartWidget, DTINI, DTFIM, true);
			stripeWidget.getField('DTINI').label = dashboardWidget.startDate;
			stripeWidget.getField('DTFIM').label = dashboardWidget.endDate;
		}
		else {
			ScreenService.showMessage('A data inicial não pode ser superior à data final.');
		}
	};

	//Nutritional tells if the function is called by storeNutriValue (nutritional = true) or nutriReportConsumer (nutritional = false)
	this.limitProductsByPeriod = function(firstWidget, secondWidget, DTINI, DTFIM, nutritional){
		LoginRepository.findAll().then(function(loginData){
			let CDFILIAL = loginData[0].CDFILIAL;
			let CDCONSUMIDOR = loginData[0].CDCONSUMIDOR;
			let CDCLIENTE = loginData[0].CDCLIENTE;

			if(nutritional){
				self.getNutritionalData(firstWidget, secondWidget, DTINI, DTFIM, CDFILIAL);
			} else {
				self.getUserShoppingData(firstWidget, secondWidget, DTINI, DTFIM, CDFILIAL, CDCLIENTE, CDCONSUMIDOR);
			}
		});
	};

	/*Nutritional Report - Store Consumption*/
	this.startFilterForStoreConsumption = function(chartWidget){
		LoginRepository.findAll().then(filterStoreConsumption.bind(null, chartWidget));
	};

	var filterStoreConsumption = function(chartWidget, loginData){
		var CDFILIAL = loginData[0].CDFILIAL;

		var dashboardWidget = chartWidget.parent;

		self.setDefaultPeriod(dashboardWidget);
		var DTINI = dashboardWidget.startDate;
		var DTFIM = dashboardWidget.endDate;

		self.getStoreConsumptionData(chartWidget, DTINI, DTFIM, CDFILIAL);
	};

	self.getStoreConsumptionData = function(chartWidget, DTINI, DTFIM, CDFILIAL){
		self.resetDynamicWidgets(chartWidget.parent, 1);
		var dashboardWidget = templateManager.container.getWidget('dashboardStoreConsumption');
		var topSellingProductsChart = templateManager.container.getWidget('topSellingProductsChart');

		NutriReportService.getNutritionalData(CDFILIAL, DTINI, DTFIM, 'consumption').then(function(nutritionalData){
			nutritionalData = nutritionalData[0];
			if (!nutritionalData.error){
					// @todo work with N families
					// --- Works with one family
					nutritionalData = nutritionalData.data[0];
					if(!nutritionalData.productsSold.error){
						topSellingProductsChart.dataSource.data = nutritionalData.productsSold.chartData;

						let gridWidget = self.buildGridWidget(nutritionalData.productsSold.gridData, chartWidget);
						gridWidget = metaDataFactory.widgetFactory(gridWidget);
						dashboardWidget.widgets.push(gridWidget);

						topSellingProductsChart.isVisible = true;
					} else {
						topSellingProductsChart.dataSource.data = [];
						topSellingProductsChart.isVisible = false;
						ScreenService.showMessage('Não há dados de consumo para o período selecionado.');
					}
			}
            else{
				    chartWidget.isVisible  = false;
					ScreenService.showMessage(nutritionalData.message);
			}
		});
	};

	this.buildGridWidget = function(productsSold, widget){
		productsSold.forEach(function(product, index){
			product.position = index + 1;
			product.position = product.position + '°';

			product.quantity = product.quantity.toFixed(2) + '%';
		});

		var fields = [
			{
				"name": "position",
				"align": "left",
				"isVisible": true
			},
			{
				"name": "NMPRODUTO",
				"align": "center",
				"isVisible": true
			},
			{
				"name": "quantity",
				"align": "right",
				"isVisible": true
			}
		];

		var gridWidget = {
			"id": "grid-widget",
			"container": widget.container,
			"parentWidget": widget.parent,
			"name": "gridWidget",
			"template": "../../../../custom-templates/templates/top-selling-grid.html",
			"isVisible": true,
			"fields": fields,
			"dataSource": {
				"data": productsSold
			},
			"width": "100%",
            "height": 1
		};

		return gridWidget;
	};

	this.calculateGridTop = function(){

		var gridElement = document.getElementById('masonry-item-grid-widget');
		if(gridElement !== null){
			var parentElement = gridElement.parentElement;
			var numberOfElements = parentElement.children.length;
			var secondToLastElement = parentElement.children[numberOfElements - 2];
			var top = parseFloat(secondToLastElement.style.top.replace('px', '')) + 200;

			gridElement.style.top = top + "px";
		}
	};

	this.setPeriodConsumption = function(chartWidget, DTINI, DTFIM){
		var stripeWidget = templateManager.container.getWidget('stripe');
		var dashboardWidget = templateManager.container.getWidget('dashboardStoreConsumption');
		var dtIni = moment(DTINI, 'DD/MM/YYYY');
		var dtFim = moment(DTFIM, 'DD/MM/YYYY');

		if (dtFim.diff(dtIni) >= 0){
			dashboardWidget.startDate = DTINI;
			dashboardWidget.endDate = DTFIM;
			ScreenService.closePopup();
			stripeWidget.activate();
			self.limitConsumptionByPeriod(chartWidget, DTINI, DTFIM);
			stripeWidget.getField('DTINI').label = dashboardWidget.startDate;
			stripeWidget.getField('DTFIM').label = dashboardWidget.endDate;
		}
		else {
			ScreenService.showMessage('A data inicial não pode ser superior à data final.');
		}
	};

	this.limitConsumptionByPeriod = function(chartWidget, DTINI, DTFIM){
		LoginRepository.findAll().then(function(loginData){
			let CDFILIAL = loginData[0].CDFILIAL;
			let CDCONSUMIDOR = loginData[0].CDCONSUMIDOR;
			let CDCLIENTE = loginData[0].CDCLIENTE;

			self.getStoreConsumptionData(chartWidget, DTINI, DTFIM, CDFILIAL);
		});
	}

	/*Nutritional Report - User specific*/
	this.startFilterForShoppingData = function(nutritionalComparisonWidget, mainChoicesWidget){
		LoginRepository.findAll().then(self.filterShoppingData.bind(null, nutritionalComparisonWidget, mainChoicesWidget));
	};

	this.filterShoppingData = function(nutritionalComparisonWidget, mainChoicesWidget, loginData){
		var CDFILIAL = loginData[0].CDFILIAL;
		var CDCLIENTE = loginData[0].CDCLIENTE;
		var CDCONSUMIDOR = loginData[0].CDCONSUMIDOR;

		var dashboardWidget = nutritionalComparisonWidget.parent;

		self.setDefaultPeriod(dashboardWidget);
		var DTINI = dashboardWidget.startDate;
		var DTFIM = dashboardWidget.endDate;

		self.getUserShoppingData(nutritionalComparisonWidget, mainChoicesWidget, DTINI, DTFIM, CDFILIAL, CDCLIENTE, CDCONSUMIDOR);
	};

	this.getUserShoppingData = function(nutritionalComparisonWidget, mainChoicesWidget, DTINI, DTFIM, CDFILIAL, CDCLIENTE, CDCONSUMIDOR){
		NutriReportService.getUserShoppingData(CDFILIAL, CDCLIENTE, CDCONSUMIDOR, DTINI, DTFIM).then(function(shoppingData){
			shoppingData = shoppingData[0];
			var userDataError = false;
			var generalDataError = false;
			var data = [];

			if(!shoppingData.error){
				// @todo with with N families
				// works with one family
				shoppingData = shoppingData.data[0];
				if(!shoppingData.userData.error){
					let userData = {
						'name': student,
						'generalPercentage': shoppingData.userData.comparisonData.bothPercentage.toFixed(0) + '%',
						'mediumPercentage': shoppingData.userData.comparisonData.mediumNutritionalValue
					};

					data.push(userData);
					mainChoicesWidget.isVisible = true;
					mainChoicesWidget.dataSource.data = shoppingData.userData.mainChoices;
				} else {
					userDataError = true;
					mainChoicesWidget.isVisible = false;
					ScreenService.showMessage('O aluno não consumiu nada no período selecionado.');
				}

				if(!shoppingData.generalData.error){
					let generalData = {
						'name': store,
						'generalPercentage': shoppingData.generalData.comparisonData.bothPercentage.toFixed(0) + '%',
						'mediumPercentage': shoppingData.generalData.comparisonData.mediumNutritionalValue
					};

					data.push(generalData);
				} else {
					generalDataError = true;
					if(!userDataError) ScreenService.showMessage('Não houve consumo na lanchonete para o período selecionado.');
				}

				if(userDataError && generalDataError){
					ScreenService.showMessage("Nenhum dado encontrado para o período selecionado.");
				}

				nutritionalComparisonWidget.dataSource.data = data;
			}

			templateManager.updateTemplate();
		});
	};

	this.setPeriodShopping = function(nutritionalComparisonWidget, mainChoicesWidget, DTINI, DTFIM){
		var stripeWidget = templateManager.container.getWidget('stripe');
		var dashboardWidget = templateManager.container.getWidget('dashboardNutriVision');
		var dtIni = moment(DTINI, 'DD/MM/YYYY');
		var dtFim = moment(DTFIM, 'DD/MM/YYYY');

		if (dtFim.diff(dtIni) >= 0){
			dashboardWidget.startDate = DTINI;
			dashboardWidget.endDate = DTFIM;
			ScreenService.closePopup();
			stripeWidget.activate();
			self.limitProductsByPeriod(nutritionalComparisonWidget, mainChoicesWidget, DTINI, DTFIM, false);
			stripeWidget.getField('DTINI').label = dashboardWidget.startDate;
			stripeWidget.getField('DTFIM').label = dashboardWidget.endDate;
		}
		else {
			ScreenService.showMessage('A data inicial não pode ser superior à data final.');
		}
	};

	this.closePopup = function(stripe){
		ScreenService.closePopup();
		stripe.activate();
	}
};

Configuration(function(ContextRegister) {
	ContextRegister.register('NutriReportController', NutriReportController);
});