function RegisterController(templateManager, ScreenService, RegisterService, Utilities){
    
    var self = this;
    
    this.showAddUpdateSonPopup = function(widget, row = {}){
        RegisterService.setOpenRow(row);
        widget.currentRow = row;
        ScreenService.openPopup(widget);
    };

    this.addSon = function(sonData, sonGridWidget){
        var newSon = {};
        newSon.NMCONSUMIDOR = sonData.NMCONSUMIDOR;
        newSon.CDIDCONSUMID = sonData.CDIDCONSUMID;
        
        if(newSon.NMCONSUMIDOR === '' || !newSon.NMCONSUMIDOR){
            ScreenService.showMessage("Informe o nome do dependente");
        } else if(newSon.CDIDCONSUMID === '' || !newSon.CDIDCONSUMID){
            ScreenService.showMessage("Informe o registro academico do dependente");
        } else{
            var add = true;
            sonGridWidget.dataSource.data.forEach(function(son){
                if(son.NMCONSUMIDOR === newSon.NMCONSUMIDOR || son.CDIDCONSUMID === newSon.CDIDCONSUMID){
                    ScreenService.showMessage("Os dados inseridos já pertemcem a outro dependente");
                    add = false;
                }
            });
            
            if(add){
                sonGridWidget.dataSource.data.push(newSon);
            
                templateManager.updateTemplate();
                ScreenService.closePopup();
                sonGridWidget.activate();
            }
        }
    };
    
    this.deleteSon = function(sonGridWidget){
        var openRow = RegisterService.getOpenRow();
        ScreenService.confirmMessage("Deseja realmente excluir o dependente?", "question")
        .then(function(){
            sonGridWidget.dataSource.data.forEach(function(son){
                if(son.NMCONSUMIDOR === openRow.NMCONSUMIDOR && son.CDIDCONSUMID === openRow.CDIDCONSUMID){
                    sonGridWidget.dataSource.data.splice(sonGridWidget.dataSource.data.indexOf(son), 1);
                }
            });
            templateManager.updateTemplate();
            ScreenService.closePopup();
            sonGridWidget.activate(); 
        });
    };
    
    this.updateSon = function(sonData, sonGridWidget){
        if(sonData.NMCONSUMIDOR === '' || !sonData.NMCONSUMIDOR){
            ScreenService.showMessage("Informe o nome do dependente");
        } 
        
        else if(sonData.CDIDCONSUMID === '' || !sonData.CDIDCONSUMID){
            ScreenService.showMessage("Informe o registro academico do dependente");
        } 
        
        else{
            templateManager.updateTemplate();
            ScreenService.closePopup();
            sonGridWidget.activate();
        }
    };

    this.cancelUpdateSon = function(sonGridWidget){
        var openRow = RegisterService.getOpenRow();
        sonGridWidget.currentRow.NMCONSUMIDOR = openRow.NMCONSUMIDOR;
        sonGridWidget.currentRow.CDIDCONSUMID = openRow.CDIDCONSUMID;

        templateManager.updateTemplate();
        ScreenService.closePopup();
        sonGridWidget.activate();
    };
    
    this.registerUser = function(dataWidget, addressWidget, sonGridWidget){
        var userData = self.manageUserData(dataWidget.currentRow, addressWidget.currentRow);
        var sonData = self.manageSonsData(sonGridWidget.dataSource.data);
        
        if(dataWidget.isValid() && addressWidget.isValid()){
            if (this.validateUserDetails(userData, sonData)) {
    		    RegisterService.registerUser(userData, sonData)
			    .then(function(response){
			        ScreenService.showMessage(response[0].message);
			        if(response[0].error === false){
			            ScreenService.openWindow('login');
			        }
			    });
    		}   
        } else{
            ScreenService.showMessage("Existem campos obrigatórios não preenchidos. Favor verificar os campos em todas as abas do cadastro novamente.");
        }
    };
    
    this.successRegister = function (response) {
        ScreenService.showMessage('Usuário criado com sucesso!');
		ScreenService.openWindow('login');
	};

	this.errorRegister = function (error) {
		ScreenService.showMessage(error.data.error.split('Exception: ')[1]);
	};
    
    this.manageUserData = function(dataWidgetData, addressWidgetData){
        var userData = {};
        var userDataArray = [];
        
        userData.CDCLIENTE = dataWidgetData['CDCLIENTE'];
        userData.CDFILIAL = dataWidgetData['CDFILIAL'];
        userData.NMLOGINCONS = dataWidgetData['NMLOGINCONS'];
        userData.CDSENHACONSMD5 = dataWidgetData['CDSENHACONSMD5'];
        userData.PASSWORD = dataWidgetData['PASSWORD'];
        userData.NMRESPCONS = dataWidgetData['NMRESPCONS'];
        userData.NRCPFRESPCON = dataWidgetData['NRCPFRESPCON'];
        userData.DSEMAILCONS = dataWidgetData['DSEMAILCONS'];
        userData.NRTELECONS = dataWidgetData['NRTELECONS'];
        userData.NRCELULARCONS = dataWidgetData['NRCELULARCONS'];
        
        userData.DSENDECONS = addressWidgetData['DSENDECONS'];
        userData.NRENDECONS = addressWidgetData['NRENDECONS'];
        userData.DSCOMPLENDECONS = addressWidgetData['DSCOMPLENDECONS'];
        userData.DSREFERENDECONS = addressWidgetData['DSREFERENDECONS'];
        userData.NRCEPCONS = addressWidgetData['NRCEPCONS'];
        userData.CDPAIS = addressWidgetData['CDPAIS'];
        userData.SGESTADO = addressWidgetData['SGESTADO'];
        userData.NMESTADO = addressWidgetData['NMESTADO'];
        userData.NMMUNICIPIO = addressWidgetData['NMMUNICIPIO'];
        userData.CDMUNICIPIO = addressWidgetData['CDMUNICIPIO'];
        userData.NMBAIRCONS = addressWidgetData['NMBAIRCONS'];
        userData.CDBAIRRO = addressWidgetData['CDBAIRRO'];
        
        userDataArray.push(userData);
        
        return userDataArray;
    };
    
    this.manageSonsData = function(sonGridWidgetData){
        var sonDataArray = [];
        
        sonGridWidgetData.forEach(function(son){
            if (son['NMCONSUMIDOR']){
                sonDataArray.push(son);
            }
        });
        
        return sonDataArray;
    }
    
    this.validateUserDetails = function (userData, sonData) {
		var validationResult = true;
		if (!Utilities.isValidCPF(userData[0].NRCPFRESPCON)) {
			ScreenService.showMessage('CPF inválido.');
			validationResult = null;
		} else if (!Utilities.isValidEmail(userData[0].DSEMAILCONS)) {
			ScreenService.showMessage('E-mail inválido.');
			validationResult = null;
		} else if (userData[0].CDSENHACONSMD5 !== userData[0].PASSWORD) {
			ScreenService.showMessage('As senhas de acesso ao App não são iguais.');
			validationResult = null;
		} else if (sonData.length === 0){
		    ScreenService.showMessage('É necessário adicionar pelo menos um dependente');
		    validationResult = null;
		}
		return validationResult;
    };
    
    this.clearRows = function (dataWidget, addressWidget, sonGridWidget){
        dataWidget.currentRow = {};
        addressWidget.currentRow = {};
        sonGridWidget.dataSource.data = [];

        templateManager.updateTemplate();
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('RegisterController', RegisterController);
});